@@ -4,6 +4,7 @@ import java.io.IOException;
 import java.math.BigInteger;
 
 import org.bouncycastle.util.Arrays;
+import org.bouncycastle.util.Properties;
 
 /**
  * Class representing the ASN.1 ENUMERATED type.
@@ -99,13 +100,9 @@ public class ASN1Enumerated
     public ASN1Enumerated(
         byte[]   bytes)
     {
-        if (bytes.length > 1)
+        if (!Properties.isOverrideSet("org.bouncycastle.asn1.allow_unsafe_integer"))
         {
-            if (bytes[0] == 0 && (bytes[1] & 0x80) == 0)
-            {
-                throw new IllegalArgumentException("malformed enumerated");
-            }
-            if (bytes[0] == (byte)0xff && (bytes[1] & 0x80) != 0)
+            if (ASN1Integer.isMalformed(bytes))
             {
                 throw new IllegalArgumentException("malformed enumerated");
             }
