@@ -11,7 +11,7 @@ import org.bouncycastle.util.encoders.Hex;
 
 
 /**
- * table of the current named curves defined in X.962 EC-DSA.
+ * Table of the current named curves defined in X.962 EC-DSA.
  */
 public class X962NamedCurves
 {
@@ -23,14 +23,14 @@ public class X962NamedCurves
             BigInteger h = BigInteger.valueOf(1);
 
             ECCurve cFp192v1 = new ECCurve.Fp(
-                new BigInteger("6277101735386680763835789423207666416083908700390324961279"),
+                new BigInteger("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFFFFFFFFFFFF", 16),
                 new BigInteger("fffffffffffffffffffffffffffffffefffffffffffffffc", 16),
                 new BigInteger("64210519e59c80e70fa7e9ab72243049feb8deecc146b9b1", 16),
                 n, h);
 
             return new X9ECParameters(
                 cFp192v1,
-                cFp192v1.decodePoint(
+                new X9ECPoint(cFp192v1,
                     Hex.decode("03188da80eb03090f67cbf20eb43a18800f4ff0afd82ff1012")),
                 n, h,
                 Hex.decode("3045AE6FC8422f64ED579528D38120EAE12196D5"));
@@ -45,14 +45,14 @@ public class X962NamedCurves
             BigInteger h = BigInteger.valueOf(1);
 
             ECCurve cFp192v2 = new ECCurve.Fp(
-                new BigInteger("6277101735386680763835789423207666416083908700390324961279"),
+                new BigInteger("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFFFFFFFFFFFF", 16),
                 new BigInteger("fffffffffffffffffffffffffffffffefffffffffffffffc", 16),
                 new BigInteger("cc22d6dfb95c6b25e49c0d6364a4e5980c393aa21668d953", 16),
                 n, h);
 
             return new X9ECParameters(
                 cFp192v2,
-                cFp192v2.decodePoint(
+                new X9ECPoint(cFp192v2,
                     Hex.decode("03eea2bae7e1497842f2de7769cfe9c989c072ad696f48034a")),
                 n, h,
                 Hex.decode("31a92ee2029fd10d901b113e990710f0d21ac6b6"));
@@ -67,14 +67,14 @@ public class X962NamedCurves
             BigInteger h = BigInteger.valueOf(1);
 
             ECCurve cFp192v3 = new ECCurve.Fp(
-                new BigInteger("6277101735386680763835789423207666416083908700390324961279"),
+                new BigInteger("FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFFFFFFFFFFFFFFFF", 16),
                 new BigInteger("fffffffffffffffffffffffffffffffefffffffffffffffc", 16),
                 new BigInteger("22123dc2395a05caa7423daeccc94760a7d462256bd56916", 16),
                 n, h);
 
             return new X9ECParameters(
                 cFp192v3,
-                cFp192v3.decodePoint(
+                new X9ECPoint(cFp192v3,
                     Hex.decode("027d29778100c65a1da1783716588dce2b8b4aee8e228f1896")),
                 n, h,
                 Hex.decode("c469684435deb378c4b65ca9591e2a5763059a2e"));
@@ -96,7 +96,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 cFp239v1,
-                cFp239v1.decodePoint(
+                new X9ECPoint(cFp239v1,
                     Hex.decode("020ffa963cdca8816ccc33b8642bedf905c3d358573d3f27fbbd3b3cb9aaaf")),
                 n, h,
                 Hex.decode("e43bb460f0b80cc0c0b075798e948060f8321b7d"));
@@ -118,7 +118,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 cFp239v2,
-                cFp239v2.decodePoint(
+                new X9ECPoint(cFp239v2,
                     Hex.decode("0238af09d98727705120c921bb5e9e26296a3cdcf2f35757a0eafd87b830e7")),
                 n, h,
                 Hex.decode("e8b4011604095303ca3b8099982be09fcb9ae616"));
@@ -140,7 +140,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 cFp239v3,
-                cFp239v3.decodePoint(
+                new X9ECPoint(cFp239v3,
                     Hex.decode("036768ae8e18bb92cfcf005c949aa2c6d94853d0e660bbf854b1c9505fe95a")),
                 n, h,
                 Hex.decode("7d7374168ffe3471b60a857686a19475d3bfa2ff"));
@@ -162,7 +162,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 cFp256v1,
-                cFp256v1.decodePoint(
+                new X9ECPoint(cFp256v1,
                     Hex.decode("036b17d1f2e12c4247f8bce6e563a440f277037d812deb33a0f4a13945d898c296")),
                 n, h,
                 Hex.decode("c49d360886e704936a6678e1139d26b7819f7e90"));
@@ -188,7 +188,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 c2m163v1,
-                c2m163v1.decodePoint(
+                new X9ECPoint(c2m163v1,
                     Hex.decode("0307AF69989546103D79329FCC3D74880F33BBE803CB")),
                 c2m163v1n, c2m163v1h,
                 Hex.decode("D2C0FB15760860DEF1EEF4D696E6768756151754"));
@@ -211,7 +211,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 c2m163v2,
-                c2m163v2.decodePoint(
+                new X9ECPoint(c2m163v2,
                     Hex.decode("030024266E4EB5106D0A964D92C4860E2671DB9B6CC5")),
                 c2m163v2n, c2m163v2h,
                 null);
@@ -234,7 +234,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 c2m163v3,
-                c2m163v3.decodePoint(
+                new X9ECPoint(c2m163v3,
                     Hex.decode("0202F9F87B7C574D0BDECF8A22E6524775F98CDEBDCB")),
                 c2m163v3n, c2m163v3h,
                 null);
@@ -257,7 +257,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 c2m176w1,
-                c2m176w1.decodePoint(
+                new X9ECPoint(c2m176w1,
                     Hex.decode("038D16C2866798B600F9F08BB4A8E860F3298CE04A5798")),
                 c2m176w1n, c2m176w1h,
                 null);
@@ -280,7 +280,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 c2m191v1,
-                c2m191v1.decodePoint(
+                new X9ECPoint(c2m191v1,
                     Hex.decode("0236B3DAF8A23206F9C4F299D7B21A9C369137F2C84AE1AA0D")),
                 c2m191v1n, c2m191v1h,
                 Hex.decode("4E13CA542744D696E67687561517552F279A8C84"));
@@ -303,7 +303,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 c2m191v2,
-                c2m191v2.decodePoint(
+                new X9ECPoint(c2m191v2,
                     Hex.decode("023809B2B7CC1B28CC5A87926AAD83FD28789E81E2C9E3BF10")),
                 c2m191v2n, c2m191v2h,
                 null);
@@ -326,7 +326,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 c2m191v3,
-                c2m191v3.decodePoint(
+                new X9ECPoint(c2m191v3,
                     Hex.decode("03375D4CE24FDE434489DE8746E71786015009E66E38A926DD")),
                 c2m191v3n, c2m191v3h,
                 null);
@@ -349,7 +349,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 c2m208w1,
-                c2m208w1.decodePoint(
+                new X9ECPoint(c2m208w1,
                     Hex.decode("0289FDFBE4ABE193DF9559ECF07AC0CE78554E2784EB8C1ED1A57A")),
                 c2m208w1n, c2m208w1h,
                 null);
@@ -372,7 +372,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 c2m239v1,
-                c2m239v1.decodePoint(
+                new X9ECPoint(c2m239v1,
                     Hex.decode("0257927098FA932E7C0A96D3FD5B706EF7E5F5C156E16B7E7C86038552E91D")),
                 c2m239v1n, c2m239v1h,
                 null);
@@ -395,7 +395,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 c2m239v2,
-                c2m239v2.decodePoint(
+                new X9ECPoint(c2m239v2,
                     Hex.decode("0228F9D04E900069C8DC47A08534FE76D2B900B7D7EF31F5709F200C4CA205")),
                 c2m239v2n, c2m239v2h,
                 null);
@@ -418,7 +418,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 c2m239v3,
-                c2m239v3.decodePoint(
+                new X9ECPoint(c2m239v3,
                     Hex.decode("0370F6E9D04D289C4E89913CE3530BFDE903977D42B146D539BF1BDE4E9C92")),
                 c2m239v3n, c2m239v3h,
                 null);
@@ -441,7 +441,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 c2m272w1,
-                c2m272w1.decodePoint(
+                new X9ECPoint(c2m272w1,
                     Hex.decode("026108BABB2CEEBCF787058A056CBE0CFE622D7723A289E08A07AE13EF0D10D171DD8D")),
                 c2m272w1n, c2m272w1h,
                 null);
@@ -464,7 +464,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 c2m304w1,
-                c2m304w1.decodePoint(
+                new X9ECPoint(c2m304w1,
                     Hex.decode("02197B07845E9BE2D96ADB0F5F3C7F2CFFBD7A3EB8B6FEC35C7FD67F26DDF6285A644F740A2614")),
                 c2m304w1n, c2m304w1h,
                 null);
@@ -487,7 +487,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 c2m359v1,
-                c2m359v1.decodePoint(
+                new X9ECPoint(c2m359v1,
                     Hex.decode("033C258EF3047767E7EDE0F1FDAA79DAEE3841366A132E163ACED4ED2401DF9C6BDCDE98E8E707C07A2239B1B097")),
                 c2m359v1n, c2m359v1h,
                 null);
@@ -510,7 +510,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 c2m368w1,
-                c2m368w1.decodePoint(
+                new X9ECPoint(c2m368w1,
                     Hex.decode("021085E2755381DCCCE3C1557AFA10C2F0C0C2825646C5B34A394CBCFA8BC16B22E7E789E927BE216F02E1FB136A5F")),
                 c2m368w1n, c2m368w1h,
                 null);
@@ -533,7 +533,7 @@ public class X962NamedCurves
 
             return new X9ECParameters(
                 c2m431r1,
-                c2m431r1.decodePoint(
+                new X9ECPoint(c2m431r1,
                     Hex.decode("02120FC05D3C67A99DE161D2F4092622FECA701BE4F50F4758714E8A87BBF2A658EF8C21E7C5EFE965361F6C2999C0C247B0DBD70CE6B7")),
                 c2m431r1n, c2m431r1h,
                 null);
