@@ -5,7 +5,7 @@ import java.security.SecureRandom;
 
 import org.bouncycastle.crypto.CipherParameters;
 import org.bouncycastle.crypto.CryptoServicesRegistrar;
-import org.bouncycastle.crypto.DSA;
+import org.bouncycastle.crypto.DSAExt;
 import org.bouncycastle.crypto.params.ECDomainParameters;
 import org.bouncycastle.crypto.params.ECKeyParameters;
 import org.bouncycastle.crypto.params.ECPrivateKeyParameters;
@@ -16,12 +16,13 @@ import org.bouncycastle.math.ec.ECConstants;
 import org.bouncycastle.math.ec.ECMultiplier;
 import org.bouncycastle.math.ec.ECPoint;
 import org.bouncycastle.math.ec.FixedPointCombMultiplier;
+import org.bouncycastle.util.BigIntegers;
 
 /**
  * GOST R 34.10-2001 Signature Algorithm
  */
 public class ECGOST3410Signer
-    implements DSA
+    implements DSAExt
 {
     ECKeyParameters key;
 
@@ -52,6 +53,11 @@ public class ECGOST3410Signer
         }
     }
 
+    public BigInteger getOrder()
+    {
+        return key.getParameters().getN();
+    }
+
     /**
      * generate a signature for the given message using the key we were
      * initialised with. For conventional GOST3410 the message should be a GOST3411
@@ -85,7 +91,7 @@ public class ECGOST3410Signer
             {
                 do
                 {
-                    k = new BigInteger(n.bitLength(), random);
+                    k = BigIntegers.createRandomBigInteger(n.bitLength(), random);
                 }
                 while (k.equals(ECConstants.ZERO));
 
