@@ -283,6 +283,24 @@ public class DTLSServerProtocol
 
         handshake.finish();
 
+//        {
+//            state.sessionParameters = new SessionParameters.Builder()
+//                .setCipherSuite(securityParameters.getCipherSuite())
+//                .setCompressionAlgorithm(securityParameters.getCompressionAlgorithm())
+//                .setExtendedMasterSecret(securityParameters.isExtendedMasterSecret())
+//                .setMasterSecret(securityParameters.getMasterSecret())
+//                .setPeerCertificate(state.clientCertificate)
+//                .setPSKIdentity(securityParameters.getPSKIdentity())
+//                .setSRPIdentity(securityParameters.getSRPIdentity())
+//                // TODO Consider filtering extensions that aren't relevant to resumed sessions
+//                .setServerExtensions(state.serverExtensions)
+//                .build();
+//
+//            state.tlsSession = TlsUtils.importSession(state.tlsSession.getSessionID(), state.sessionParameters);
+//
+//            state.serverContext.setResumableSession(state.tlsSession);
+//        }
+
         state.server.notifyHandshakeComplete();
 
         return new DTLSTransport(recordLayer);
@@ -364,7 +382,7 @@ public class DTLSServerProtocol
         TlsUtils.writeUint16(selectedCipherSuite, buf);
         TlsUtils.writeUint8(selectedCompressionMethod, buf);
 
-        state.serverExtensions = state.server.getServerExtensions();
+        state.serverExtensions = TlsExtensionsUtils.ensureExtensionsInitialised(state.server.getServerExtensions());
 
         /*
          * RFC 5746 3.6. Server Behavior: Initial Handshake
@@ -388,15 +406,13 @@ public class DTLSServerProtocol
                  * If the secure_renegotiation flag is set to TRUE, the server MUST include an empty
                  * "renegotiation_info" extension in the ServerHello message.
                  */
-                state.serverExtensions = TlsExtensionsUtils.ensureExtensionsInitialised(state.serverExtensions);
                 state.serverExtensions.put(TlsProtocol.EXT_RenegotiationInfo,
                     TlsProtocol.createRenegotiationInfo(TlsUtils.EMPTY_BYTES));
             }
         }
 
-        if (securityParameters.extendedMasterSecret)
+        if (securityParameters.isExtendedMasterSecret())
         {
-            state.serverExtensions = TlsExtensionsUtils.ensureExtensionsInitialised(state.serverExtensions);
             TlsExtensionsUtils.addExtendedMasterSecretExtension(state.serverExtensions);
         }
 
@@ -406,7 +422,7 @@ public class DTLSServerProtocol
          * extensions.
          */
 
-        if (state.serverExtensions != null)
+        if (!state.serverExtensions.isEmpty())
         {
             securityParameters.encryptThenMAC = TlsExtensionsUtils.hasEncryptThenMACExtension(state.serverExtensions);
 
@@ -623,12 +639,18 @@ public class DTLSServerProtocol
         SecurityParameters securityParameters = context.getSecurityParameters();
 
         /*
-         * TODO[session-hash]
-         * 
-         * draft-ietf-tls-session-hash-04 4. Clients and servers SHOULD NOT accept handshakes
-         * that do not use the extended master secret [..]. (and see 5.2, 5.3)
+         * TODO[resumption] Check RFC 7627 5.4. for required behaviour 
+         */
+
+        /*
+         * RFC 7627 4. Clients and servers SHOULD NOT accept handshakes that do not use the extended
+         * master secret [..]. (and see 5.2, 5.3)
          */
         securityParameters.extendedMasterSecret = TlsExtensionsUtils.hasExtendedMasterSecretExtension(state.clientExtensions);
+        if (!securityParameters.isExtendedMasterSecret() && state.server.requiresExtendedMasterSecret())
+        {
+            throw new TlsFatalAlert(AlertDescription.handshake_failure);
+        }
 
         context.setClientVersion(client_version);
 
