@@ -1,14 +1,26 @@
 package org.bouncycastle.math.ec.rfc8032;
 
+import java.security.SecureRandom;
+
+import org.bouncycastle.crypto.Digest;
 import org.bouncycastle.crypto.digests.SHA512Digest;
+import org.bouncycastle.math.ec.rfc7748.X25519;
 import org.bouncycastle.math.ec.rfc7748.X25519Field;
 import org.bouncycastle.math.raw.Interleave;
 import org.bouncycastle.math.raw.Nat;
 import org.bouncycastle.math.raw.Nat256;
 import org.bouncycastle.util.Arrays;
+import org.bouncycastle.util.Strings;
 
 public abstract class Ed25519
 {
+    public static final class Algorithm
+    {
+        public static final int Ed25519 = 0;
+        public static final int Ed25519ctx = 1;
+        public static final int Ed25519ph = 2;
+    }
+
     private static final long M28L = 0x0FFFFFFFL;
     private static final long M32L = 0xFFFFFFFFL;
 
@@ -16,11 +28,12 @@ public abstract class Ed25519
     private static final int SCALAR_INTS = 8;
     private static final int SCALAR_BYTES = SCALAR_INTS * 4;
 
+    public static final int PREHASH_SIZE = 64;
     public static final int PUBLIC_KEY_SIZE = POINT_BYTES;
     public static final int SECRET_KEY_SIZE = 32;
     public static final int SIGNATURE_SIZE = POINT_BYTES + SCALAR_BYTES;
 
-//    private static final byte[] DOM2_PREFIX = Strings.toByteArray("SigEd25519 no Ed25519 collisions");
+    private static final byte[] DOM2_PREFIX = Strings.toByteArray("SigEd25519 no Ed25519 collisions");
 
     private static final int[] P = new int[]{ 0xFFFFFFED, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0x7FFFFFFF };
     private static final int[] L = new int[]{ 0x5CF5D3ED, 0x5812631A, 0xA2F79CD6, 0x14DEF9DE, 0x00000000, 0x00000000, 0x00000000, 0x10000000 };
@@ -50,10 +63,20 @@ public abstract class Ed25519
     private static final int PRECOMP_POINTS = 1 << (PRECOMP_TEETH - 1);
     private static final int PRECOMP_MASK = PRECOMP_POINTS - 1;
 
+    private static Object precompLock = new Object();
     // TODO[ed25519] Convert to PointPrecomp
     private static PointExt[] precompBaseTable = null;
     private static int[] precompBase = null;
 
+    private static class PointAccum
+    {
+        int[] x = X25519Field.create();
+        int[] y = X25519Field.create();
+        int[] z = X25519Field.create();
+        int[] u = X25519Field.create();
+        int[] v = X25519Field.create();
+    }
+
     private static class PointExt
     {
         int[] x = X25519Field.create();
@@ -85,6 +108,12 @@ public abstract class Ed25519
         return reduceScalar(result);
     }
 
+    private static boolean checkContextVar(byte[] ctx , byte phflag)
+    {
+        return ctx == null && phflag == 0x00 
+            || ctx != null && ctx.length < 256;
+    }
+
     private static boolean checkPointVar(byte[] p)
     {
         int[] t = new int[8];
@@ -100,6 +129,16 @@ public abstract class Ed25519
         return !Nat256.gte(n, L);
     }
 
+    private static Digest createDigest()
+    {
+        return new SHA512Digest();
+    }
+
+    public static Digest createPrehash()
+    {
+        return createDigest();
+    }
+
     private static int decode24(byte[] bs, int off)
     {
         int n = bs[  off] & 0xFF;
@@ -171,6 +210,17 @@ public abstract class Ed25519
         decode32(k, kOff, n, 0, SCALAR_INTS);
     }
 
+    private static void dom2(Digest d, byte phflag, byte[] ctx)
+    {
+        if (ctx != null)
+        {
+            d.update(DOM2_PREFIX, 0, DOM2_PREFIX.length);
+            d.update(phflag);
+            d.update((byte)ctx.length);
+            d.update(ctx, 0, ctx.length);
+        }
+    }
+
     private static void encode24(int n, byte[] bs, int off)
     {
         bs[  off] = (byte)(n       );
@@ -192,7 +242,7 @@ public abstract class Ed25519
         encode24((int)(n >>> 32), bs, off + 4);
     }
 
-    private static void encodePoint(PointExt p, byte[] r, int rOff)
+    private static void encodePoint(PointAccum p, byte[] r, int rOff)
     {
         int[] x = X25519Field.create();
         int[] y = X25519Field.create();
@@ -207,9 +257,14 @@ public abstract class Ed25519
         r[rOff + POINT_BYTES - 1] |= ((x[0] & 1) << 7);
     }
 
+    public static void generatePrivateKey(SecureRandom random, byte[] k)
+    {
+        random.nextBytes(k);
+    }
+
     public static void generatePublicKey(byte[] sk, int skOff, byte[] pk, int pkOff)
     {
-        SHA512Digest d = new SHA512Digest();
+        Digest d = createDigest();
         byte[] h = new byte[d.getDigestSize()];
 
         d.update(sk, skOff, SECRET_KEY_SIZE);
@@ -274,8 +329,10 @@ public abstract class Ed25519
         return ws;
     }
 
-    private static void implSign(SHA512Digest d, byte[] h, byte[] s, byte[] pk, int pkOff, byte[] m, int mOff, int mLen, byte[] sig, int sigOff)
+    private static void implSign(Digest d, byte[] h, byte[] s, byte[] pk, int pkOff, byte[] ctx, byte phflag,
+        byte[] m, int mOff, int mLen, byte[] sig, int sigOff)
     {
+        dom2(d, phflag, ctx);
         d.update(h, SCALAR_BYTES, SCALAR_BYTES);
         d.update(m, mOff, mLen);
         d.doFinal(h, 0);
@@ -284,8 +341,9 @@ public abstract class Ed25519
         byte[] R = new byte[POINT_BYTES];
         scalarMultBaseEncoded(r, R, 0);
 
+        dom2(d, phflag, ctx);
         d.update(R, 0, POINT_BYTES);
-        d.update(pk, 0, POINT_BYTES);
+        d.update(pk, pkOff, POINT_BYTES);
         d.update(m, mOff, mLen);
         d.doFinal(h, 0);
 
@@ -296,7 +354,140 @@ public abstract class Ed25519
         System.arraycopy(S, 0, sig, sigOff + POINT_BYTES, SCALAR_BYTES);
     }
 
-    private static void pointAddVar(boolean negate, PointExt p, PointExt r)
+    private static void implSign(byte[] sk, int skOff, byte[] ctx, byte phflag, byte[] m, int mOff, int mLen,
+        byte[] sig, int sigOff)
+    {
+        if (!checkContextVar(ctx, phflag))
+        {
+            throw new IllegalArgumentException("ctx");
+        }
+
+        Digest d = createDigest();
+        byte[] h = new byte[d.getDigestSize()];
+
+        d.update(sk, skOff, SECRET_KEY_SIZE);
+        d.doFinal(h, 0);
+
+        byte[] s = new byte[SCALAR_BYTES];
+        pruneScalar(h, 0, s);
+
+        byte[] pk = new byte[POINT_BYTES];
+        scalarMultBaseEncoded(s, pk, 0);
+
+        implSign(d, h, s, pk, 0, ctx, phflag, m, mOff, mLen, sig, sigOff);
+    }
+
+    private static void implSign(byte[] sk, int skOff, byte[] pk, int pkOff, byte[] ctx, byte phflag,
+        byte[] m, int mOff, int mLen, byte[] sig, int sigOff)
+    {
+        if (!checkContextVar(ctx, phflag))
+        {
+            throw new IllegalArgumentException("ctx");
+        }
+
+        Digest d = createDigest();
+        byte[] h = new byte[d.getDigestSize()];
+
+        d.update(sk, skOff, SECRET_KEY_SIZE);
+        d.doFinal(h, 0);
+
+        byte[] s = new byte[SCALAR_BYTES];
+        pruneScalar(h, 0, s);
+
+        implSign(d, h, s, pk, pkOff, ctx, phflag, m, mOff, mLen, sig, sigOff);
+    }
+
+    private static boolean implVerify(byte[] sig, int sigOff, byte[] pk, int pkOff, byte[] ctx, byte phflag,
+        byte[] m, int mOff, int mLen)
+    {
+        if (!checkContextVar(ctx, phflag))
+        {
+            throw new IllegalArgumentException("ctx");
+        }
+
+        byte[] R = Arrays.copyOfRange(sig, sigOff, sigOff + POINT_BYTES);
+        byte[] S = Arrays.copyOfRange(sig, sigOff + POINT_BYTES, sigOff + SIGNATURE_SIZE);
+
+        if (!checkPointVar(R))
+        {
+            return false;
+        }
+        if (!checkScalarVar(S))
+        {
+            return false;
+        }
+
+        PointExt pA = new PointExt();
+        if (!decodePointVar(pk, pkOff, true, pA))
+        {
+            return false;
+        }
+
+        Digest d = createDigest();
+        byte[] h = new byte[d.getDigestSize()];
+
+        dom2(d, phflag, ctx);
+        d.update(R, 0, POINT_BYTES);
+        d.update(pk, pkOff, POINT_BYTES);
+        d.update(m, mOff, mLen);
+        d.doFinal(h, 0);
+
+        byte[] k = reduceScalar(h);
+
+        int[] nS = new int[SCALAR_INTS];
+        decodeScalar(S, 0, nS);
+
+        int[] nA = new int[SCALAR_INTS];
+        decodeScalar(k, 0, nA);
+
+        PointAccum pR = new PointAccum();
+        scalarMultStraussVar(nS, nA, pA, pR);
+
+        byte[] check = new byte[POINT_BYTES];
+        encodePoint(pR, check, 0);
+
+        return Arrays.areEqual(check, R);
+    }
+
+    private static void pointAddVar(boolean negate, PointExt p, PointAccum r)
+    {
+        int[] A = X25519Field.create();
+        int[] B = X25519Field.create();
+        int[] C = X25519Field.create();
+        int[] D = X25519Field.create();
+        int[] E = r.u;
+        int[] F = X25519Field.create();
+        int[] G = X25519Field.create();
+        int[] H = r.v;
+
+        int[] c, d, f, g;
+        if (negate)
+        {
+            c = D; d = C; f = G; g = F;
+        }
+        else
+        {
+            c = C; d = D; f = F; g = G;
+        }
+
+        X25519Field.apm(r.y, r.x, B, A);
+        X25519Field.apm(p.y, p.x, d, c);
+        X25519Field.mul(A, C, A);
+        X25519Field.mul(B, D, B);
+        X25519Field.mul(r.u, r.v, C);
+        X25519Field.mul(C, p.t, C);
+        X25519Field.mul(C, C_d2, C);
+        X25519Field.mul(r.z, p.z, D);
+        X25519Field.add(D, D, D);
+        X25519Field.apm(B, A, H, E);
+        X25519Field.apm(D, C, g, f);
+        X25519Field.carry(g);
+        X25519Field.mul(E, F, r.x);
+        X25519Field.mul(G, H, r.y);
+        X25519Field.mul(F, G, r.z);
+    }
+
+    private static void pointAddVar(boolean negate, PointExt p, PointExt q, PointExt r)
     {
         int[] A = X25519Field.create();
         int[] B = X25519Field.create();
@@ -317,13 +508,13 @@ public abstract class Ed25519
             c = C; d = D; f = F; g = G;
         }
 
-        X25519Field.apm(r.y, r.x, B, A);
-        X25519Field.apm(p.y, p.x, d, c);
+        X25519Field.apm(p.y, p.x, B, A);
+        X25519Field.apm(q.y, q.x, d, c);
         X25519Field.mul(A, C, A);
         X25519Field.mul(B, D, B);
-        X25519Field.mul(r.t, p.t, C);
+        X25519Field.mul(p.t, q.t, C);
         X25519Field.mul(C, C_d2, C);
-        X25519Field.mul(r.z, p.z, D);
+        X25519Field.mul(p.z, q.z, D);
         X25519Field.add(D, D, D);
         X25519Field.apm(B, A, H, E);
         X25519Field.apm(D, C, g, f);
@@ -334,27 +525,37 @@ public abstract class Ed25519
         X25519Field.mul(E, H, r.t);
     }
 
-    private static void pointAddPrecomp(PointPrecomp p, PointExt r)
+    private static void pointAddPrecomp(PointPrecomp p, PointAccum r)
     {
         int[] A = X25519Field.create();
         int[] B = X25519Field.create();
         int[] C = X25519Field.create();
-        int[] E = X25519Field.create();
+        int[] E = r.u;
         int[] F = X25519Field.create();
         int[] G = X25519Field.create();
-        int[] H = X25519Field.create();
+        int[] H = r.v;
 
         X25519Field.apm(r.y, r.x, B, A);
         X25519Field.mul(A, p.ymx_h, A);
         X25519Field.mul(B, p.ypx_h, B);
-        X25519Field.mul(r.t, p.xyd, C);
+        X25519Field.mul(r.u, r.v, C);
+        X25519Field.mul(C, p.xyd, C);
         X25519Field.apm(B, A, H, E);
         X25519Field.apm(r.z, C, G, F);
         X25519Field.carry(G);
         X25519Field.mul(E, F, r.x);
         X25519Field.mul(G, H, r.y);
         X25519Field.mul(F, G, r.z);
-        X25519Field.mul(E, H, r.t);
+    }
+
+    private static PointExt pointCopy(PointAccum p)
+    {
+        PointExt r = new PointExt();
+        X25519Field.copy(p.x, 0, r.x, 0);
+        X25519Field.copy(p.y, 0, r.y, 0);
+        X25519Field.copy(p.z, 0, r.z, 0);
+        X25519Field.mul(p.u, p.v, r.t);
+        return r;
     }
 
     private static PointExt pointCopy(PointExt p)
@@ -367,15 +568,15 @@ public abstract class Ed25519
         return r;
     }
 
-    private static void pointDouble(PointExt r)
+    private static void pointDouble(PointAccum r)
     {
         int[] A = X25519Field.create();
         int[] B = X25519Field.create();
         int[] C = X25519Field.create();
-        int[] E = X25519Field.create();
+        int[] E = r.u;
         int[] F = X25519Field.create();
         int[] G = X25519Field.create();
-        int[] H = X25519Field.create();
+        int[] H = r.v;
 
         X25519Field.sqr(r.x, A);
         X25519Field.sqr(r.y, B);
@@ -390,7 +591,13 @@ public abstract class Ed25519
         X25519Field.mul(E, F, r.x);
         X25519Field.mul(G, H, r.y);
         X25519Field.mul(F, G, r.z);
-        X25519Field.mul(E, H, r.t);
+    }
+
+    private static void pointExtendXY(PointAccum p)
+    {
+        X25519Field.one(p.z);
+        X25519Field.copy(p.x, 0, p.u, 0);
+        X25519Field.copy(p.y, 0, p.v, 0);
     }
 
     private static void pointExtendXY(PointExt p)
@@ -417,19 +624,29 @@ public abstract class Ed25519
 
     private static PointExt[] pointPrecompVar(PointExt p, int count)
     {
-        PointExt d = pointCopy(p);
-        pointDouble(d);
+//        assert count > 0;
+
+        PointExt d = new PointExt();
+        pointAddVar(false, p, p, d);
 
         PointExt[] table = new PointExt[count];
         table[0] = pointCopy(p);
         for (int i = 1; i < count; ++i)
         {
-            table[i] = pointCopy(table[i - 1]);
-            pointAddVar(false, d, table[i]);
+            pointAddVar(false, table[i - 1], d, table[i] = new PointExt());
         }
         return table;
     }
 
+    private static void pointSetNeutral(PointAccum p)
+    {
+        X25519Field.zero(p.x);
+        X25519Field.one(p.y);
+        X25519Field.one(p.z);
+        X25519Field.zero(p.u);
+        X25519Field.one(p.v);
+    }
+
     private static void pointSetNeutral(PointExt p)
     {
         X25519Field.zero(p.x);
@@ -438,88 +655,102 @@ public abstract class Ed25519
         X25519Field.zero(p.t);
     }
 
-    public synchronized static void precompute()
+    public static void precompute()
     {
-        if (precompBase != null)
+        synchronized (precompLock)
         {
-            return;
-        }
-
-        PointExt p = new PointExt();
-        X25519Field.copy(B_x, 0, p.x, 0);
-        X25519Field.copy(B_y, 0, p.y, 0);
-        pointExtendXY(p);
-
-        precompBaseTable = pointPrecompVar(p, 1 << (WNAF_WIDTH_BASE - 2));
-
-        precompBase = new int[PRECOMP_BLOCKS * PRECOMP_POINTS * 3 * X25519Field.SIZE];
-
-        int off = 0;
-        for (int b = 0; b < PRECOMP_BLOCKS; ++b)
-        {
-            PointExt[] ds = new PointExt[PRECOMP_TEETH];
-
-            PointExt sum = new PointExt();
-            pointSetNeutral(sum);
-
-            for (int t = 0; t < PRECOMP_TEETH; ++t)
+            if (precompBase != null)
             {
-                pointAddVar(true, p, sum);
-                pointDouble(p);
+                return;
+            }
 
-                ds[t] = pointCopy(p);
+            // Precomputed table for the base point in verification ladder
+            {
+                PointExt b = new PointExt();
+                X25519Field.copy(B_x, 0, b.x, 0);
+                X25519Field.copy(B_y, 0, b.y, 0);
+                pointExtendXY(b);
 
-                for (int s = 1; s < PRECOMP_SPACING; ++s)
+                precompBaseTable = pointPrecompVar(b, 1 << (WNAF_WIDTH_BASE - 2));
+            }
+
+            PointAccum p = new PointAccum();
+            X25519Field.copy(B_x, 0, p.x, 0);
+            X25519Field.copy(B_y, 0, p.y, 0);
+            pointExtendXY(p);
+
+            precompBase = new int[PRECOMP_BLOCKS * PRECOMP_POINTS * 3 * X25519Field.SIZE];
+
+            int off = 0;
+            for (int b = 0; b < PRECOMP_BLOCKS; ++b)
+            {
+                PointExt[] ds = new PointExt[PRECOMP_TEETH];
+
+                PointExt sum = new PointExt();
+                pointSetNeutral(sum);
+
+                for (int t = 0; t < PRECOMP_TEETH; ++t)
                 {
+                    PointExt q = pointCopy(p);
+                    pointAddVar(true, sum, q, sum);
                     pointDouble(p);
+
+                    ds[t] = pointCopy(p);
+
+                    if (b + t != PRECOMP_BLOCKS + PRECOMP_TEETH - 2)
+                    {
+                        for (int s = 1; s < PRECOMP_SPACING; ++s)
+                        {
+                            pointDouble(p);
+                        }
+                    }
                 }
-            }
 
-            PointExt[] points = new PointExt[PRECOMP_POINTS];
-            int k = 0;
-            points[k++] = sum;
+                PointExt[] points = new PointExt[PRECOMP_POINTS];
+                int k = 0;
+                points[k++] = sum;
 
-            for (int t = 0; t < (PRECOMP_TEETH - 1); ++t)
-            {
-                int size = 1 << t;
-                for (int j = 0; j < size; ++j)
+                for (int t = 0; t < (PRECOMP_TEETH - 1); ++t)
                 {
-                    points[k] = pointCopy(points[k - size]);
-                    pointAddVar(false, ds[t], points[k++]);
+                    int size = 1 << t;
+                    for (int j = 0; j < size; ++j, ++k)
+                    {
+                        pointAddVar(false, points[k - size], ds[t], points[k] = new PointExt());
+                    }
+                }
+
+//                assert k == PRECOMP_POINTS;
+
+                for (int i = 0; i < PRECOMP_POINTS; ++i)
+                {
+                    PointExt q = points[i];
+
+                    int[] x = X25519Field.create();
+                    int[] y = X25519Field.create();
+
+                    X25519Field.add(q.z, q.z, x);
+                    // TODO[ed25519] Batch inversion
+                    X25519Field.inv(x, y);
+                    X25519Field.mul(q.x, y, x);
+                    X25519Field.mul(q.y, y, y);
+
+                    PointPrecomp r = new PointPrecomp();
+                    X25519Field.apm(y, x, r.ypx_h, r.ymx_h);
+                    X25519Field.mul(x, y, r.xyd);
+                    X25519Field.mul(r.xyd, C_d4, r.xyd);
+
+                    X25519Field.normalize(r.ypx_h);
+                    X25519Field.normalize(r.ymx_h);
+//                    X25519Field.normalize(r.xyd);
+
+                    X25519Field.copy(r.ypx_h, 0, precompBase, off);    off += X25519Field.SIZE;
+                    X25519Field.copy(r.ymx_h, 0, precompBase, off);    off += X25519Field.SIZE;
+                    X25519Field.copy(r.xyd,   0, precompBase, off);    off += X25519Field.SIZE;
                 }
             }
 
-//            assert k == POINTS;
-
-            for (int i = 0; i < PRECOMP_POINTS; ++i)
-            {
-                PointExt q = points[i];
-
-                int[] x = X25519Field.create();
-                int[] y = X25519Field.create();
-
-                X25519Field.add(q.z, q.z, x);
-                // TODO[ed25519] Batch inversion
-                X25519Field.inv(x, y);
-                X25519Field.mul(q.x, y, x);
-                X25519Field.mul(q.y, y, y);
-
-                PointPrecomp r = new PointPrecomp();
-                X25519Field.apm(y, x, r.ypx_h, r.ymx_h);
-                X25519Field.mul(x, y, r.xyd);
-                X25519Field.mul(r.xyd, C_d4, r.xyd);
-
-                X25519Field.normalize(r.ypx_h);
-                X25519Field.normalize(r.ymx_h);
-//                X25519Field.normalize(r.xyd);
-
-                X25519Field.copy(r.ypx_h, 0, precompBase, off);    off += X25519Field.SIZE;
-                X25519Field.copy(r.ymx_h, 0, precompBase, off);    off += X25519Field.SIZE;
-                X25519Field.copy(r.xyd,   0, precompBase, off);    off += X25519Field.SIZE;
-            }
+//            assert off == precompBase.length;
         }
-
-//        assert off == precompBase.length;
     }
 
     private static void pruneScalar(byte[] n, int nOff, byte[] r)
@@ -667,7 +898,7 @@ public abstract class Ed25519
         return r;
     }
 
-    private static void scalarMultBase(byte[] k, PointExt r)
+    private static void scalarMultBase(byte[] k, PointAccum r)
     {
         precompute();
 
@@ -722,12 +953,31 @@ public abstract class Ed25519
 
     private static void scalarMultBaseEncoded(byte[] k, byte[] r, int rOff)
     {
-        PointExt p = new PointExt();
+        PointAccum p = new PointAccum();
         scalarMultBase(k, p);
         encodePoint(p, r, rOff);
     }
 
-    private static void scalarMultStraussVar(int[] nb, int[] np, PointExt p, PointExt r)
+    /**
+     * NOTE: Only for use by X25519
+     */
+    public static void scalarMultBaseYZ(X25519.Friend friend, byte[] k, int kOff, int[] y, int[] z)
+    {
+        if (null == friend)
+        {
+            throw new NullPointerException("This method is only for use by X25519");
+        }
+
+        byte[] n = new byte[SCALAR_BYTES];
+        pruneScalar(k, kOff, n);
+
+        PointAccum p = new PointAccum();
+        scalarMultBase(n, p);
+        X25519Field.copy(p.y, 0, y, 0);
+        X25519Field.copy(p.z, 0, z, 0);
+    }
+
+    private static void scalarMultStraussVar(int[] nb, int[] np, PointExt p, PointAccum r)
     {
         precompute();
 
@@ -777,77 +1027,106 @@ public abstract class Ed25519
 
     public static void sign(byte[] sk, int skOff, byte[] m, int mOff, int mLen, byte[] sig, int sigOff)
     {
-        SHA512Digest d = new SHA512Digest();
-        byte[] h = new byte[d.getDigestSize()];
+        byte[] ctx = null;
+        byte phflag = 0x00;
 
-        d.update(sk, skOff, SECRET_KEY_SIZE);
-        d.doFinal(h, 0);
-
-        byte[] s = new byte[SCALAR_BYTES];
-        pruneScalar(h, 0, s);
-
-        byte[] pk = new byte[POINT_BYTES];
-        scalarMultBaseEncoded(s, pk, 0);
-
-        implSign(d, h, s, pk, 0, m, mOff, mLen, sig, sigOff);
+        implSign(sk, skOff, ctx, phflag, m, mOff, mLen, sig, sigOff);
     }
 
     public static void sign(byte[] sk, int skOff, byte[] pk, int pkOff, byte[] m, int mOff, int mLen, byte[] sig, int sigOff)
     {
-        SHA512Digest d = new SHA512Digest();
-        byte[] h = new byte[d.getDigestSize()];
+        byte[] ctx = null;
+        byte phflag = 0x00;
 
-        d.update(sk, skOff, SECRET_KEY_SIZE);
-        d.doFinal(h, 0);
+        implSign(sk, skOff, pk, pkOff, ctx, phflag, m, mOff, mLen, sig, sigOff);
+    }
 
-        byte[] s = new byte[SCALAR_BYTES];
-        pruneScalar(h, 0, s);
+    public static void sign(byte[] sk, int skOff, byte[] ctx, byte[] m, int mOff, int mLen, byte[] sig, int sigOff)
+    {
+        byte phflag = 0x00;
 
-        implSign(d, h, s, pk, pkOff, m, mOff, mLen, sig, sigOff);
+        implSign(sk, skOff, ctx, phflag, m, mOff, mLen, sig, sigOff);
+    }
+
+    public static void sign(byte[] sk, int skOff, byte[] pk, int pkOff, byte[] ctx, byte[] m, int mOff, int mLen, byte[] sig, int sigOff)
+    {
+        byte phflag = 0x00;
+
+        implSign(sk, skOff, pk, pkOff, ctx, phflag, m, mOff, mLen, sig, sigOff);
+    }
+
+    public static void signPrehash(byte[] sk, int skOff, byte[] ctx, byte[] ph, int phOff, byte[] sig, int sigOff)
+    {
+        byte phflag = 0x01;
+
+        implSign(sk, skOff, ctx, phflag, ph, phOff, PREHASH_SIZE, sig, sigOff);
+    }
+
+    public static void signPrehash(byte[] sk, int skOff, byte[] pk, int pkOff, byte[] ctx, byte[] ph, int phOff, byte[] sig, int sigOff)
+    {
+        byte phflag = 0x01;
+
+        implSign(sk, skOff, pk, pkOff, ctx, phflag, ph, phOff, PREHASH_SIZE, sig, sigOff);
+    }
+
+    public static void signPrehash(byte[] sk, int skOff, byte[] ctx, Digest ph, byte[] sig, int sigOff)
+    {
+        byte[] m = new byte[PREHASH_SIZE];
+        if (PREHASH_SIZE != ph.doFinal(m, 0))
+        {
+            throw new IllegalArgumentException("ph");
+        }
+
+        byte phflag = 0x01;
+
+        implSign(sk, skOff, ctx, phflag, m, 0, m.length, sig, sigOff);
+    }
+
+    public static void signPrehash(byte[] sk, int skOff, byte[] pk, int pkOff, byte[] ctx, Digest ph, byte[] sig, int sigOff)
+    {
+        byte[] m = new byte[PREHASH_SIZE];
+        if (PREHASH_SIZE != ph.doFinal(m, 0))
+        {
+            throw new IllegalArgumentException("ph");
+        }
+
+        byte phflag = 0x01;
+
+        implSign(sk, skOff, pk, pkOff, ctx, phflag, m, 0, m.length, sig, sigOff);
     }
 
     public static boolean verify(byte[] sig, int sigOff, byte[] pk, int pkOff, byte[] m, int mOff, int mLen)
     {
-        byte[] R = Arrays.copyOfRange(sig, sigOff, sigOff + POINT_BYTES);
-        byte[] S = Arrays.copyOfRange(sig, sigOff + POINT_BYTES, sigOff + SIGNATURE_SIZE);
+        byte[] ctx = null;
+        byte phflag = 0x00;
 
-        if (!checkPointVar(R))
+        return implVerify(sig, sigOff, pk, pkOff, ctx, phflag, m, mOff, mLen);
+    }
+
+    public static boolean verify(byte[] sig, int sigOff, byte[] pk, int pkOff, byte[] ctx, byte[] m, int mOff, int mLen)
+    {
+        byte phflag = 0x00;
+
+        return implVerify(sig, sigOff, pk, pkOff, ctx, phflag, m, mOff, mLen);
+    }
+
+    public static boolean verifyPrehash(byte[] sig, int sigOff, byte[] pk, int pkOff, byte[] ctx, byte[] ph, int phOff)
+    {
+        byte phflag = 0x01;
+
+        return implVerify(sig, sigOff, pk, pkOff, ctx, phflag, ph, phOff, PREHASH_SIZE);
+    }
+
+    public static boolean verifyPrehash(byte[] sig, int sigOff, byte[] pk, int pkOff, byte[] ctx, Digest ph)
+    {
+        byte[] m = new byte[PREHASH_SIZE];
+        if (PREHASH_SIZE != ph.doFinal(m, 0))
         {
-            return false;
-        }
-        if (!checkScalarVar(S))
-        {
-            return false;
+            throw new IllegalArgumentException("ph");
         }
 
-        PointExt pA = new PointExt();
-        if (!decodePointVar(pk, pkOff, true, pA))
-        {
-            return false;
-        }
+        byte phflag = 0x01;
 
-        SHA512Digest d = new SHA512Digest();
-        byte[] h = new byte[d.getDigestSize()];
-
-        d.update(R, 0, POINT_BYTES);
-        d.update(pk, pkOff, POINT_BYTES);
-        d.update(m, mOff, mLen);
-        d.doFinal(h, 0);
-
-        byte[] k = reduceScalar(h);
-
-        int[] nS = new int[SCALAR_INTS];
-        decodeScalar(S, 0, nS);
-
-        int[] nA = new int[SCALAR_INTS];
-        decodeScalar(k, 0, nA);
-
-        PointExt pR = new PointExt();
-        scalarMultStraussVar(nS, nA, pA, pR);
-
-        byte[] check = new byte[POINT_BYTES];
-        encodePoint(pR, check, 0);
-
-        return Arrays.areEqual(check, R);
+        return implVerify(sig, sigOff, pk, pkOff, ctx, phflag, m, 0, m.length);
     }
 }
