@@ -1,6 +1,10 @@
 package org.bouncycastle.math.ec.rfc8032;
 
+import java.security.SecureRandom;
+
+import org.bouncycastle.crypto.Xof;
 import org.bouncycastle.crypto.digests.SHAKEDigest;
+import org.bouncycastle.math.ec.rfc7748.X448;
 import org.bouncycastle.math.ec.rfc7748.X448Field;
 import org.bouncycastle.math.raw.Nat;
 import org.bouncycastle.util.Arrays;
@@ -8,6 +12,12 @@ import org.bouncycastle.util.Strings;
 
 public abstract class Ed448
 {
+    public static final class Algorithm
+    {
+        public static final int Ed448 = 0;
+        public static final int Ed448ph = 1;
+    }
+
     private static final long M26L = 0x03FFFFFFL;
     private static final long M28L = 0x0FFFFFFFL;
     private static final long M32L = 0xFFFFFFFFL;
@@ -16,6 +26,7 @@ public abstract class Ed448
     private static final int SCALAR_INTS = 14;
     private static final int SCALAR_BYTES = SCALAR_INTS * 4 + 1;
 
+    public static final int PREHASH_SIZE = 64;
     public static final int PUBLIC_KEY_SIZE = POINT_BYTES;
     public static final int SECRET_KEY_SIZE = 57;
     public static final int SIGNATURE_SIZE = POINT_BYTES + SCALAR_BYTES;
@@ -59,6 +70,7 @@ public abstract class Ed448
     private static final int PRECOMP_POINTS = 1 << (PRECOMP_TEETH - 1);
     private static final int PRECOMP_MASK = PRECOMP_POINTS - 1;
 
+    private static Object precompLock = new Object();
     // TODO[ed448] Convert to PointPrecomp
     private static PointExt[] precompBaseTable = null;
     private static int[] precompBase = null;
@@ -121,6 +133,16 @@ public abstract class Ed448
         return !Nat.gte(SCALAR_INTS, n, L);
     }
 
+    public static Xof createPrehash()
+    {
+        return createXof();
+    }
+
+    private static Xof createXof()
+    {
+        return new SHAKEDigest(256);
+    }
+
     private static int decode16(byte[] bs, int off)
     {
         int n = bs[off] & 0xFF;
@@ -202,7 +224,7 @@ public abstract class Ed448
         decode32(k, kOff, n, 0, SCALAR_INTS);
     }
 
-    private static void dom4(SHAKEDigest d, byte x, byte[] y)
+    private static void dom4(Xof d, byte x, byte[] y)
     {
         d.update(DOM4_PREFIX, 0, DOM4_PREFIX.length);
         d.update(x);
@@ -246,9 +268,14 @@ public abstract class Ed448
         r[rOff + POINT_BYTES - 1] = (byte)((x[0] & 1) << 7);
     }
 
+    public static void generatePrivateKey(SecureRandom random, byte[] k)
+    {
+        random.nextBytes(k);
+    }
+
     public static void generatePublicKey(byte[] sk, int skOff, byte[] pk, int pkOff)
     {
-        SHAKEDigest d = new SHAKEDigest(256);
+        Xof d = createXof();
         byte[] h = new byte[SCALAR_BYTES * 2];
 
         d.update(sk, skOff, SECRET_KEY_SIZE);
@@ -257,7 +284,7 @@ public abstract class Ed448
         byte[] s = new byte[SCALAR_BYTES];
         pruneScalar(h, 0, s);
 
-        scalarMultBaseEncodedVar(s, pk, pkOff);
+        scalarMultBaseEncoded(s, pk, pkOff);
     }
 
     private static byte[] getWNAF(int[] n, int width)
@@ -313,10 +340,9 @@ public abstract class Ed448
         return ws;
     }
 
-    private static void implSignVar(SHAKEDigest d, byte[] h, byte[] s, byte[] pk, int pkOff, byte[] ctx, byte[] m, int mOff, int mLen, byte[] sig, int sigOff)
+    private static void implSign(Xof d, byte[] h, byte[] s, byte[] pk, int pkOff, byte[] ctx, byte phflag,
+        byte[] m, int mOff, int mLen, byte[] sig, int sigOff)
     {
-        byte phflag = 0x00;
-
         dom4(d, phflag, ctx);
         d.update(h, SCALAR_BYTES, SCALAR_BYTES);
         d.update(m, mOff, mLen);
@@ -324,7 +350,7 @@ public abstract class Ed448
 
         byte[] r = reduceScalar(h);
         byte[] R = new byte[POINT_BYTES];
-        scalarMultBaseEncodedVar(r, R, 0);
+        scalarMultBaseEncoded(r, R, 0);
 
         dom4(d, phflag, ctx);
         d.update(R, 0, POINT_BYTES);
@@ -339,6 +365,101 @@ public abstract class Ed448
         System.arraycopy(S, 0, sig, sigOff + POINT_BYTES, SCALAR_BYTES);
     }
 
+    private static void implSign(byte[] sk, int skOff, byte[] ctx, byte phflag, byte[] m, int mOff, int mLen,
+        byte[] sig, int sigOff)
+    {
+        if (!checkContextVar(ctx))
+        {
+            throw new IllegalArgumentException("ctx");
+        }
+
+        Xof d = createXof();
+        byte[] h = new byte[SCALAR_BYTES * 2];
+
+        d.update(sk, skOff, SECRET_KEY_SIZE);
+        d.doFinal(h, 0, h.length);
+
+        byte[] s = new byte[SCALAR_BYTES];
+        pruneScalar(h, 0, s);
+
+        byte[] pk = new byte[POINT_BYTES];
+        scalarMultBaseEncoded(s, pk, 0);
+
+        implSign(d, h, s, pk, 0, ctx, phflag, m, mOff, mLen, sig, sigOff);
+    }
+
+    private static void implSign(byte[] sk, int skOff, byte[] pk, int pkOff, byte[] ctx, byte phflag,
+        byte[] m, int mOff, int mLen, byte[] sig, int sigOff)
+    {
+        if (!checkContextVar(ctx))
+        {
+            throw new IllegalArgumentException("ctx");
+        }
+
+        Xof d = createXof();
+        byte[] h = new byte[SCALAR_BYTES * 2];
+
+        d.update(sk, skOff, SECRET_KEY_SIZE);
+        d.doFinal(h, 0, h.length);
+
+        byte[] s = new byte[SCALAR_BYTES];
+        pruneScalar(h, 0, s);
+
+        implSign(d, h, s, pk, pkOff, ctx, phflag, m, mOff, mLen, sig, sigOff);
+    }
+
+    private static boolean implVerify(byte[] sig, int sigOff, byte[] pk, int pkOff, byte[] ctx, byte phflag,
+        byte[] m, int mOff, int mLen)
+    {
+        if (!checkContextVar(ctx))
+        {
+            throw new IllegalArgumentException("ctx");
+        }
+
+        byte[] R = Arrays.copyOfRange(sig, sigOff, sigOff + POINT_BYTES);
+        byte[] S = Arrays.copyOfRange(sig, sigOff + POINT_BYTES, sigOff + SIGNATURE_SIZE);
+
+        if (!checkPointVar(R))
+        {
+            return false;
+        }
+        if (!checkScalarVar(S))
+        {
+            return false;
+        }
+
+        PointExt pA = new PointExt();
+        if (!decodePointVar(pk, pkOff, true, pA))
+        {
+            return false;
+        }
+
+        Xof d = createXof();
+        byte[] h = new byte[SCALAR_BYTES * 2];
+
+        dom4(d, phflag, ctx);
+        d.update(R, 0, POINT_BYTES);
+        d.update(pk, pkOff, POINT_BYTES);
+        d.update(m, mOff, mLen);
+        d.doFinal(h, 0, h.length);
+
+        byte[] k = reduceScalar(h);
+
+        int[] nS = new int[SCALAR_INTS];
+        decodeScalar(S, 0, nS);
+
+        int[] nA = new int[SCALAR_INTS];
+        decodeScalar(k, 0, nA);
+
+        PointExt pR = new PointExt();
+        scalarMultStraussVar(nS, nA, pA, pR);
+
+        byte[] check = new byte[POINT_BYTES];
+        encodePoint(pR, check, 0);
+
+        return Arrays.areEqual(check, R);
+    }
+
     private static void pointAddVar(boolean negate, PointExt p, PointExt r)
     {
         int[] A = X448Field.create();
@@ -475,6 +596,8 @@ public abstract class Ed448
 
     private static PointExt[] pointPrecompVar(PointExt p, int count)
     {
+//        assert count > 0;
+
         PointExt d = pointCopy(p);
         pointDouble(d);
 
@@ -495,85 +618,91 @@ public abstract class Ed448
         X448Field.one(p.z);
     }
 
-    public synchronized static void precompute()
+    public static void precompute()
     {
-        if (precompBase != null)
+        synchronized (precompLock)
         {
-            return;
-        }
-
-        PointExt p = new PointExt();
-        X448Field.copy(B_x, 0, p.x, 0);
-        X448Field.copy(B_y, 0, p.y, 0);
-        pointExtendXY(p);
-
-        precompBaseTable = pointPrecompVar(p, 1 << (WNAF_WIDTH_BASE - 2));
-
-        precompBase = new int[PRECOMP_BLOCKS * PRECOMP_POINTS * 2 * X448Field.SIZE];
-
-        int off = 0;
-        for (int b = 0; b < PRECOMP_BLOCKS; ++b)
-        {
-            PointExt[] ds = new PointExt[PRECOMP_TEETH];
-
-            PointExt sum = new PointExt();
-            pointSetNeutral(sum);
-
-            for (int t = 0; t < PRECOMP_TEETH; ++t)
+            if (precompBase != null)
             {
-                pointAddVar(true, p, sum);
-                pointDouble(p);
+                return;
+            }
 
-                ds[t] = pointCopy(p);
+            PointExt p = new PointExt();
+            X448Field.copy(B_x, 0, p.x, 0);
+            X448Field.copy(B_y, 0, p.y, 0);
+            pointExtendXY(p);
 
-                for (int s = 1; s < PRECOMP_SPACING; ++s)
+            precompBaseTable = pointPrecompVar(p, 1 << (WNAF_WIDTH_BASE - 2));
+
+            precompBase = new int[PRECOMP_BLOCKS * PRECOMP_POINTS * 2 * X448Field.SIZE];
+
+            int off = 0;
+            for (int b = 0; b < PRECOMP_BLOCKS; ++b)
+            {
+                PointExt[] ds = new PointExt[PRECOMP_TEETH];
+
+                PointExt sum = new PointExt();
+                pointSetNeutral(sum);
+
+                for (int t = 0; t < PRECOMP_TEETH; ++t)
                 {
+                    pointAddVar(true, p, sum);
                     pointDouble(p);
+
+                    ds[t] = pointCopy(p);
+
+                    if (b + t != PRECOMP_BLOCKS + PRECOMP_TEETH - 2)
+                    {
+                        for (int s = 1; s < PRECOMP_SPACING; ++s)
+                        {
+                            pointDouble(p);
+                        }
+                    }
                 }
-            }
 
-            PointExt[] points = new PointExt[PRECOMP_POINTS];
-            int k = 0;
-            points[k++] = sum;
+                PointExt[] points = new PointExt[PRECOMP_POINTS];
+                int k = 0;
+                points[k++] = sum;
 
-            for (int t = 0; t < (PRECOMP_TEETH - 1); ++t)
-            {
-                int size = 1 << t;
-                for (int j = 0; j < size; ++j)
+                for (int t = 0; t < (PRECOMP_TEETH - 1); ++t)
                 {
-                    points[k] = pointCopy(points[k - size]);
-                    pointAddVar(false, ds[t], points[k++]);
+                    int size = 1 << t;
+                    for (int j = 0; j < size; ++j, ++k)
+                    {
+                        points[k] = pointCopy(points[k - size]);
+                        pointAddVar(false, ds[t], points[k]);
+                    }
+                }
+
+//                assert k == PRECOMP_POINTS;
+
+                for (int i = 0; i < PRECOMP_POINTS; ++i)
+                {
+                    PointExt q = points[i];
+                    // TODO[ed448] Batch inversion
+                    X448Field.inv(q.z, q.z);
+                    X448Field.mul(q.x, q.z, q.x);
+                    X448Field.mul(q.y, q.z, q.y);
+
+//                    X448Field.normalize(q.x);
+//                    X448Field.normalize(q.y);
+
+                    X448Field.copy(q.x, 0, precompBase, off);   off += X448Field.SIZE;
+                    X448Field.copy(q.y, 0, precompBase, off);   off += X448Field.SIZE;
                 }
             }
 
-//            assert k == POINTS;
-
-            for (int i = 0; i < PRECOMP_POINTS; ++i)
-            {
-                PointExt q = points[i];
-                // TODO[ed448] Batch inversion
-                X448Field.inv(q.z, q.z);
-                X448Field.mul(q.x, q.z, q.x);
-                X448Field.mul(q.y, q.z, q.y);
-
-//                X448Field.normalize(q.x);
-//                X448Field.normalize(q.y);
-
-                X448Field.copy(q.x, 0, precompBase, off);   off += X448Field.SIZE;
-                X448Field.copy(q.y, 0, precompBase, off);   off += X448Field.SIZE;
-            }
+//            assert off == precompBase.length;
         }
-
-//        assert off == precompBase.length;
     }
 
     private static void pruneScalar(byte[] n, int nOff, byte[] r)
     {
-        System.arraycopy(n, nOff, r, 0, SCALAR_BYTES);
+        System.arraycopy(n, nOff, r, 0, SCALAR_BYTES - 1);
 
         r[0] &= 0xFC;
         r[SCALAR_BYTES - 2] |= 0x80;
-        r[SCALAR_BYTES - 1] &= 0x00;
+        r[SCALAR_BYTES - 1]  = 0x00;
     }
 
     private static byte[] reduceScalar(byte[] n)
@@ -881,8 +1010,9 @@ public abstract class Ed448
                 int w = 0;
                 for (int t = 0; t < PRECOMP_TEETH; ++t)
                 {
-                    int tBit = (n[tPos >>> 5] >>> (tPos & 0x1F)) & 1;
-                    w |= tBit << t;
+                    int tBit = n[tPos >>> 5] >>> (tPos & 0x1F);
+                    w &= ~(1 << t);
+                    w ^= (tBit << t);
                     tPos += PRECOMP_SPACING;
                 }
 
@@ -908,13 +1038,32 @@ public abstract class Ed448
         }
     }
 
-    private static void scalarMultBaseEncodedVar(byte[] k, byte[] r, int rOff)
+    private static void scalarMultBaseEncoded(byte[] k, byte[] r, int rOff)
     {
         PointExt p = new PointExt();
         scalarMultBase(k, p);
         encodePoint(p, r, rOff);
     }
 
+    /**
+     * NOTE: Only for use by X448
+     */
+    public static void scalarMultBaseXY(X448.Friend friend, byte[] k, int kOff, int[] x, int[] y)
+    {
+        if (null == friend)
+        {
+            throw new NullPointerException("This method is only for use by X448");
+        }
+
+        byte[] n = new byte[SCALAR_BYTES];
+        pruneScalar(k, kOff, n);
+
+        PointExt p = new PointExt();
+        scalarMultBase(n, p);
+        X448Field.copy(p.x, 0, x, 0);
+        X448Field.copy(p.y, 0, y, 0);
+    }
+
     private static void scalarMultStraussVar(int[] nb, int[] np, PointExt p, PointExt r)
     {
         precompute();
@@ -965,95 +1114,82 @@ public abstract class Ed448
 
     public static void sign(byte[] sk, int skOff, byte[] ctx, byte[] m, int mOff, int mLen, byte[] sig, int sigOff)
     {
-        if (!checkContextVar(ctx))
-        {
-            throw new IllegalArgumentException("ctx");
-        }
+        byte phflag = 0x00;
 
-        SHAKEDigest d = new SHAKEDigest(256);
-        byte[] h = new byte[SCALAR_BYTES * 2];
-
-        d.update(sk, skOff, SECRET_KEY_SIZE);
-        d.doFinal(h, 0, h.length);
-
-        byte[] s = new byte[SCALAR_BYTES];
-        pruneScalar(h, 0, s);
-
-        byte[] pk = new byte[POINT_BYTES];
-        scalarMultBaseEncodedVar(s, pk, 0);
-
-        implSignVar(d, h, s, pk, 0, ctx, m, mOff, mLen, sig, sigOff);
+        implSign(sk, skOff, ctx, phflag, m, mOff, mLen, sig, sigOff);
     }
 
     public static void sign(byte[] sk, int skOff, byte[] pk, int pkOff, byte[] ctx, byte[] m, int mOff, int mLen, byte[] sig, int sigOff)
     {
-        if (!checkContextVar(ctx))
+        byte phflag = 0x00;
+
+        implSign(sk, skOff, pk, pkOff, ctx, phflag, m, mOff, mLen, sig, sigOff);
+    }
+
+    public static void signPrehash(byte[] sk, int skOff, byte[] ctx, byte[] ph, int phOff, byte[] sig, int sigOff)
+    {
+        byte phflag = 0x01;
+
+        implSign(sk, skOff, ctx, phflag, ph, phOff, PREHASH_SIZE, sig, sigOff);
+    }
+
+    public static void signPrehash(byte[] sk, int skOff, byte[] pk, int pkOff, byte[] ctx, byte[] ph, int phOff, byte[] sig, int sigOff)
+    {
+        byte phflag = 0x01;
+
+        implSign(sk, skOff, pk, pkOff, ctx, phflag, ph, phOff, PREHASH_SIZE, sig, sigOff);
+    }
+
+    public static void signPrehash(byte[] sk, int skOff, byte[] ctx, Xof ph, byte[] sig, int sigOff)
+    {
+        byte[] m = new byte[PREHASH_SIZE];
+        if (PREHASH_SIZE != ph.doFinal(m, 0, PREHASH_SIZE))
         {
-            throw new IllegalArgumentException("ctx");
+            throw new IllegalArgumentException("ph");
         }
 
-        SHAKEDigest d = new SHAKEDigest(256);
-        byte[] h = new byte[SCALAR_BYTES * 2];
+        byte phflag = 0x01;
 
-        d.update(sk, skOff, SECRET_KEY_SIZE);
-        d.doFinal(h, 0, h.length);
+        implSign(sk, skOff, ctx, phflag, m, 0, m.length, sig, sigOff);
+    }
 
-        byte[] s = new byte[SCALAR_BYTES];
-        pruneScalar(h, 0, s);
+    public static void signPrehash(byte[] sk, int skOff, byte[] pk, int pkOff, byte[] ctx, Xof ph, byte[] sig, int sigOff)
+    {
+        byte[] m = new byte[PREHASH_SIZE];
+        if (PREHASH_SIZE != ph.doFinal(m, 0, PREHASH_SIZE))
+        {
+            throw new IllegalArgumentException("ph");
+        }
 
-        implSignVar(d, h, s, pk, pkOff, ctx, m, mOff, mLen, sig, sigOff);
+        byte phflag = 0x01;
+
+        implSign(sk, skOff, pk, pkOff, ctx, phflag, m, 0, m.length, sig, sigOff);
     }
 
     public static boolean verify(byte[] sig, int sigOff, byte[] pk, int pkOff, byte[] ctx, byte[] m, int mOff, int mLen)
     {
-        if (!checkContextVar(ctx))
-        {
-            throw new IllegalArgumentException("ctx");
-        }
-
-        byte[] R = Arrays.copyOfRange(sig, sigOff, sigOff + POINT_BYTES);
-        byte[] S = Arrays.copyOfRange(sig, sigOff + POINT_BYTES, sigOff + SIGNATURE_SIZE);
-
-        if (!checkPointVar(R))
-        {
-            return false;
-        }
-        if (!checkScalarVar(S))
-        {
-            return false;
-        }
-
-        PointExt pA = new PointExt();
-        if (!decodePointVar(pk, pkOff, true, pA))
-        {
-            return false;
-        }
-
         byte phflag = 0x00;
 
-        SHAKEDigest d = new SHAKEDigest(256);
-        byte[] h = new byte[SCALAR_BYTES * 2];
+        return implVerify(sig, sigOff, pk, pkOff, ctx, phflag, m, mOff, mLen);
+    }
 
-        dom4(d, phflag, ctx);
-        d.update(R, 0, POINT_BYTES);
-        d.update(pk, pkOff, POINT_BYTES);
-        d.update(m, mOff, mLen);
-        d.doFinal(h, 0, h.length);
+    public static boolean verifyPrehash(byte[] sig, int sigOff, byte[] pk, int pkOff, byte[] ctx, byte[] ph, int phOff)
+    {
+        byte phflag = 0x01;
 
-        byte[] k = reduceScalar(h);
+        return implVerify(sig, sigOff, pk, pkOff, ctx, phflag, ph, phOff, PREHASH_SIZE);
+    }
 
-        int[] nS = new int[SCALAR_INTS];
-        decodeScalar(S, 0, nS);
+    public static boolean verifyPrehash(byte[] sig, int sigOff, byte[] pk, int pkOff, byte[] ctx, Xof ph)
+    {
+        byte[] m = new byte[PREHASH_SIZE];
+        if (PREHASH_SIZE != ph.doFinal(m, 0, PREHASH_SIZE))
+        {
+            throw new IllegalArgumentException("ph");
+        }
 
-        int[] nA = new int[SCALAR_INTS];
-        decodeScalar(k, 0, nA);
+        byte phflag = 0x01;
 
-        PointExt pR = new PointExt();
-        scalarMultStraussVar(nS, nA, pA, pR);
-
-        byte[] check = new byte[POINT_BYTES];
-        encodePoint(pR, check, 0);
-
-        return Arrays.areEqual(check, R);
+        return implVerify(sig, sigOff, pk, pkOff, ctx, phflag, m, 0, m.length);
     }
 }
