@@ -238,16 +238,54 @@ public abstract class Nat256
         z[7] = x[7];
     }
 
+    public static void copy(int[] x, int xOff, int[] z, int zOff)
+    {
+        z[zOff + 0] = x[xOff + 0];
+        z[zOff + 1] = x[xOff + 1];
+        z[zOff + 2] = x[xOff + 2];
+        z[zOff + 3] = x[xOff + 3];
+        z[zOff + 4] = x[xOff + 4];
+        z[zOff + 5] = x[xOff + 5];
+        z[zOff + 6] = x[xOff + 6];
+        z[zOff + 7] = x[xOff + 7];
+    }
+
+    public static void copy64(long[] x, long[] z)
+    {
+        z[0] = x[0];
+        z[1] = x[1];
+        z[2] = x[2];
+        z[3] = x[3];
+    }
+
+    public static void copy64(long[] x, int xOff, long[] z, int zOff)
+    {
+        z[zOff + 0] = x[xOff + 0];
+        z[zOff + 1] = x[xOff + 1];
+        z[zOff + 2] = x[xOff + 2];
+        z[zOff + 3] = x[xOff + 3];
+    }
+
     public static int[] create()
     {
         return new int[8];
     }
 
+    public static long[] create64()
+    {
+        return new long[4];
+    }
+
     public static int[] createExt()
     {
         return new int[16];
     }
 
+    public static long[] createExt64()
+    {
+        return new long[8];
+    }
+
     public static boolean diff(int[] x, int xOff, int[] y, int yOff, int[] z, int zOff)
     {
         boolean pos = gte(x, xOff, y, yOff);
@@ -274,6 +312,18 @@ public abstract class Nat256
         return true;
     }
 
+    public static boolean eq64(long[] x, long[] y)
+    {
+        for (int i = 3; i >= 0; --i)
+        {
+            if (x[i] != y[i])
+            {
+                return false;
+            }
+        }
+        return true;
+    }
+
     public static int[] fromBigInteger(BigInteger x)
     {
         if (x.signum() < 0 || x.bitLength() > 256)
@@ -291,6 +341,23 @@ public abstract class Nat256
         return z;
     }
 
+    public static long[] fromBigInteger64(BigInteger x)
+    {
+        if (x.signum() < 0 || x.bitLength() > 256)
+        {
+            throw new IllegalArgumentException();
+        }
+
+        long[] z = create64();
+        int i = 0;
+        while (x.signum() != 0)
+        {
+            z[i++] = x.longValue();
+            x = x.shiftRight(64);
+        }
+        return z;
+    }
+
     public static int getBit(int[] x, int bit)
     {
         if (bit == 0)
@@ -350,6 +417,22 @@ public abstract class Nat256
         return true;
     }
 
+    public static boolean isOne64(long[] x)
+    {
+        if (x[0] != 1L)
+        {
+            return false;
+        }
+        for (int i = 1; i < 4; ++i)
+        {
+            if (x[i] != 0L)
+            {
+                return false;
+            }
+        }
+        return true;
+    }
+
     public static boolean isZero(int[] x)
     {
         for (int i = 0; i < 8; ++i)
@@ -362,6 +445,18 @@ public abstract class Nat256
         return true;
     }
 
+    public static boolean isZero64(long[] x)
+    {
+        for (int i = 0; i < 4; ++i)
+        {
+            if (x[i] != 0L)
+            {
+                return false;
+            }
+        }
+        return true;
+    }
+
     public static void mul(int[] x, int[] y, int[] zz)
     {
         long y_0 = y[0] & M;
@@ -851,8 +946,8 @@ public abstract class Nat256
         }
 
         long x_3 = x[3] & M;
-        long zz_5 = zz[5] & M;
-        long zz_6 = zz[6] & M;
+        long zz_5 = (zz[5] & M) + (zz_4 >>> 32); zz_4 &= M;
+        long zz_6 = (zz[6] & M) + (zz_5 >>> 32); zz_5 &= M;
         {
             zz_3 += x_3 * x_0;
             w = (int)zz_3;
@@ -866,8 +961,8 @@ public abstract class Nat256
         }
 
         long x_4 = x[4] & M;
-        long zz_7 = zz[7] & M;
-        long zz_8 = zz[8] & M;
+        long zz_7 = (zz[7] & M) + (zz_6 >>> 32); zz_6 &= M;
+        long zz_8 = (zz[8] & M) + (zz_7 >>> 32); zz_7 &= M;
         {
             zz_4 += x_4 * x_0;
             w = (int)zz_4;
@@ -883,8 +978,8 @@ public abstract class Nat256
         }
 
         long x_5 = x[5] & M;
-        long zz_9 = zz[9] & M;
-        long zz_10 = zz[10] & M;
+        long zz_9 = (zz[9] & M) + (zz_8 >>> 32); zz_8 &= M;
+        long zz_10 = (zz[10] & M) + (zz_9 >>> 32); zz_9 &= M;
         {
             zz_5 += x_5 * x_0;
             w = (int)zz_5;
@@ -902,8 +997,8 @@ public abstract class Nat256
         }
 
         long x_6 = x[6] & M;
-        long zz_11 = zz[11] & M;
-        long zz_12 = zz[12] & M;
+        long zz_11 = (zz[11] & M) + (zz_10 >>> 32); zz_10 &= M;
+        long zz_12 = (zz[12] & M) + (zz_11 >>> 32); zz_11 &= M;
         {
             zz_6 += x_6 * x_0;
             w = (int)zz_6;
@@ -923,8 +1018,8 @@ public abstract class Nat256
         }
 
         long x_7 = x[7] & M;
-        long zz_13 = zz[13] & M;
-        long zz_14 = zz[14] & M;
+        long zz_13 = (zz[13] & M) + (zz_12 >>> 32); zz_12 &= M;
+        long zz_14 = (zz[14] & M) + (zz_13 >>> 32); zz_13 &= M;
         {
             zz_7 += x_7 * x_0;
             w = (int)zz_7;
@@ -960,7 +1055,7 @@ public abstract class Nat256
         w = (int)zz_14;
         zz[14] = (w << 1) | c;
         c = w >>> 31;
-        w = zz[15] + (int)(zz_14 >> 32);
+        w = zz[15] + (int)(zz_14 >>> 32);
         zz[15] = (w << 1) | c;
     }
 
@@ -1015,8 +1110,8 @@ public abstract class Nat256
         }
 
         long x_3 = x[xOff + 3] & M;
-        long zz_5 = zz[zzOff + 5] & M;
-        long zz_6 = zz[zzOff + 6] & M;
+        long zz_5 = (zz[zzOff + 5] & M) + (zz_4 >>> 32); zz_4 &= M;
+        long zz_6 = (zz[zzOff + 6] & M) + (zz_5 >>> 32); zz_5 &= M;
         {
             zz_3 += x_3 * x_0;
             w = (int)zz_3;
@@ -1030,8 +1125,8 @@ public abstract class Nat256
         }
 
         long x_4 = x[xOff + 4] & M;
-        long zz_7 = zz[zzOff + 7] & M;
-        long zz_8 = zz[zzOff + 8] & M;
+        long zz_7 = (zz[zzOff + 7] & M) + (zz_6 >>> 32); zz_6 &= M;
+        long zz_8 = (zz[zzOff + 8] & M) + (zz_7 >>> 32); zz_7 &= M;
         {
             zz_4 += x_4 * x_0;
             w = (int)zz_4;
@@ -1047,8 +1142,8 @@ public abstract class Nat256
         }
 
         long x_5 = x[xOff + 5] & M;
-        long zz_9 = zz[zzOff + 9] & M;
-        long zz_10 = zz[zzOff + 10] & M;
+        long zz_9 = (zz[zzOff + 9] & M) + (zz_8 >>> 32); zz_8 &= M;
+        long zz_10 = (zz[zzOff + 10] & M) + (zz_9 >>> 32); zz_9 &= M;
         {
             zz_5 += x_5 * x_0;
             w = (int)zz_5;
@@ -1066,8 +1161,8 @@ public abstract class Nat256
         }
 
         long x_6 = x[xOff + 6] & M;
-        long zz_11 = zz[zzOff + 11] & M;
-        long zz_12 = zz[zzOff + 12] & M;
+        long zz_11 = (zz[zzOff + 11] & M) + (zz_10 >>> 32); zz_10 &= M;
+        long zz_12 = (zz[zzOff + 12] & M) + (zz_11 >>> 32); zz_11 &= M;
         {
             zz_6 += x_6 * x_0;
             w = (int)zz_6;
@@ -1087,8 +1182,8 @@ public abstract class Nat256
         }
 
         long x_7 = x[xOff + 7] & M;
-        long zz_13 = zz[zzOff + 13] & M;
-        long zz_14 = zz[zzOff + 14] & M;
+        long zz_13 = (zz[zzOff + 13] & M) + (zz_12 >>> 32); zz_12 &= M;
+        long zz_14 = (zz[zzOff + 14] & M) + (zz_13 >>> 32); zz_13 &= M;
         {
             zz_7 += x_7 * x_0;
             w = (int)zz_7;
@@ -1124,7 +1219,7 @@ public abstract class Nat256
         w = (int)zz_14;
         zz[zzOff + 14] = (w << 1) | c;
         c = w >>> 31;
-        w = zz[zzOff + 15] + (int)(zz_14 >> 32);
+        w = zz[zzOff + 15] + (int)(zz_14 >>> 32);
         zz[zzOff + 15] = (w << 1) | c;
     }
 
@@ -1292,6 +1387,20 @@ public abstract class Nat256
         return new BigInteger(1, bs);
     }
 
+    public static BigInteger toBigInteger64(long[] x)
+    {
+        byte[] bs = new byte[32];
+        for (int i = 0; i < 4; ++i)
+        {
+            long x_i = x[i];
+            if (x_i != 0L)
+            {
+                Pack.longToBigEndian(x_i, bs, (3 - i) << 3);
+            }
+        }
+        return new BigInteger(1, bs);
+    }
+
     public static void zero(int[] z)
     {
         z[0] = 0;
