@@ -43,7 +43,7 @@ import org.bouncycastle.crypto.modes.CCMBlockCipher;
 import org.bouncycastle.crypto.modes.CFBBlockCipher;
 import org.bouncycastle.crypto.modes.CTSBlockCipher;
 import org.bouncycastle.crypto.modes.EAXBlockCipher;
-import org.bouncycastle.crypto.modes.GCMBlockCipher;
+import org.bouncycastle.crypto.modes.NISTCTSBlockCipher;
 import org.bouncycastle.crypto.modes.OCBBlockCipher;
 import org.bouncycastle.crypto.modes.OFBBlockCipher;
 import org.bouncycastle.crypto.modes.SICBlockCipher;
@@ -72,7 +72,7 @@ public class CipherStreamTest
         testReadWrite(cipher, params, false);
         testReadWrite(cipher, params, true);
 
-        if (!(cipher instanceof CTSBlockCipher))
+        if (!(cipher instanceof CTSBlockCipher || cipher instanceof NISTCTSBlockCipher))
         {
             testWriteReadEmpty(cipher, params, false);
             testWriteReadEmpty(cipher, params, true);
@@ -539,6 +539,12 @@ public class CipherStreamTest
         {
             testMode(new CTSBlockCipher(cipher1), key);
         }
+        if (blockSize <= 16 && streamSize >= blockSize)
+        {
+            testMode(new NISTCTSBlockCipher(NISTCTSBlockCipher.CS1, cipher1), key);
+            testMode(new NISTCTSBlockCipher(NISTCTSBlockCipher.CS2, cipher1), key);
+            testMode(new NISTCTSBlockCipher(NISTCTSBlockCipher.CS3, cipher1), key);
+        }
         if (blockSize == 8 || blockSize == 16)
         {
             testMode(new EAXBlockCipher(cipher1), withIv);
@@ -546,7 +552,8 @@ public class CipherStreamTest
         if (blockSize == 16)
         {
             testMode(new CCMBlockCipher(cipher1), new ParametersWithIV(key, new byte[7]));
-            testMode(new GCMBlockCipher(cipher1), withIv);
+            // TODO: need to have a GCM safe version of testMode.
+//            testMode(new GCMBlockCipher(cipher1), withIv);
             testMode(new OCBBlockCipher(cipher1, cipher2), new ParametersWithIV(key, new byte[15]));
         }
     }
