@@ -9,9 +9,11 @@ public class RegressionTest
         new GMSSSignerTest(),
         new McElieceFujisakiCipherTest(),
         new McElieceKobaraImaiCipherTest(),
-        new McEliecePKCSCipherTest(),
+        new McElieceCipherTest(),
         new McEliecePointchevalCipherTest(),
-        new RainbowSignerTest()
+        new RainbowSignerTest() ,
+        new Sphincs256Test(),
+        new NewHopeTest()
     };
 
     public static void main(
