@@ -28,6 +28,7 @@ import org.bouncycastle.asn1.cryptopro.CryptoProObjectIdentifiers;
 import org.bouncycastle.asn1.nist.NISTObjectIdentifiers;
 import org.bouncycastle.asn1.oiw.OIWObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
+import org.bouncycastle.asn1.rosstandart.RosstandartObjectIdentifiers;
 import org.bouncycastle.asn1.teletrust.TeleTrusTObjectIdentifiers;
 import org.bouncycastle.asn1.x9.X9ObjectIdentifiers;
 import org.bouncycastle.cms.CMSAlgorithm;
@@ -95,6 +96,8 @@ public class SMIMESignedGenerator
     public static final String  ENCRYPTION_RSA_PSS = PKCSObjectIdentifiers.id_RSASSA_PSS.getId();
     public static final String  ENCRYPTION_GOST3410 = CryptoProObjectIdentifiers.gostR3410_94.getId();
     public static final String  ENCRYPTION_ECGOST3410 = CryptoProObjectIdentifiers.gostR3410_2001.getId();
+    public static final String  ENCRYPTION_ECGOST3410_2012_256 = RosstandartObjectIdentifiers.id_tc26_gost_3410_12_256.getId();
+    public static final String  ENCRYPTION_ECGOST3410_2012_512 = RosstandartObjectIdentifiers.id_tc26_gost_3410_12_512.getId();
 
     private static final String CERTIFICATE_MANAGEMENT_CONTENT = "application/pkcs7-mime; name=smime.p7c; smime-type=certs-only";
     private static final String DETACHED_SIGNATURE_TYPE = "application/pkcs7-signature; name=smime.p7s; smime-type=signed-data";
@@ -104,10 +107,8 @@ public class SMIMESignedGenerator
     public static final Map RFC5751_MICALGS;
     public static final Map STANDARD_MICALGS;
 
-    private static MailcapCommandMap addCommands(CommandMap cm)
+    private static MailcapCommandMap addCommands(MailcapCommandMap mc)
     {
-        MailcapCommandMap mc = (MailcapCommandMap)cm;
-
         mc.addMailcap("application/pkcs7-signature;; x-java-content-handler=org.bouncycastle.mail.smime.handlers.pkcs7_signature");
         mc.addMailcap("application/pkcs7-mime;; x-java-content-handler=org.bouncycastle.mail.smime.handlers.pkcs7_mime");
         mc.addMailcap("application/x-pkcs7-signature;; x-java-content-handler=org.bouncycastle.mail.smime.handlers.x_pkcs7_signature");
@@ -123,7 +124,12 @@ public class SMIMESignedGenerator
         {
             public Object run()
             {
-                CommandMap.setDefaultCommandMap(addCommands(CommandMap.getDefaultCommandMap()));
+                CommandMap commandMap = CommandMap.getDefaultCommandMap();
+
+                if (commandMap instanceof MailcapCommandMap)
+                {
+                    CommandMap.setDefaultCommandMap(addCommands((MailcapCommandMap)commandMap));
+                }
 
                 return null;
             }
@@ -138,6 +144,8 @@ public class SMIMESignedGenerator
         stdMicAlgs.put(CMSAlgorithm.SHA384, "sha-384");
         stdMicAlgs.put(CMSAlgorithm.SHA512, "sha-512");
         stdMicAlgs.put(CMSAlgorithm.GOST3411, "gostr3411-94");
+        stdMicAlgs.put(CMSAlgorithm.GOST3411_2012_256, "gostr3411-2012-256");
+        stdMicAlgs.put(CMSAlgorithm.GOST3411_2012_512, "gostr3411-2012-512");
 
         RFC5751_MICALGS = Collections.unmodifiableMap(stdMicAlgs);
 
@@ -150,6 +158,8 @@ public class SMIMESignedGenerator
         oldMicAlgs.put(CMSAlgorithm.SHA384, "sha384");
         oldMicAlgs.put(CMSAlgorithm.SHA512, "sha512");
         oldMicAlgs.put(CMSAlgorithm.GOST3411, "gostr3411-94");
+        oldMicAlgs.put(CMSAlgorithm.GOST3411_2012_256, "gostr3411-2012-256");
+        oldMicAlgs.put(CMSAlgorithm.GOST3411_2012_512, "gostr3411-2012-512");
 
         RFC3851_MICALGS = Collections.unmodifiableMap(oldMicAlgs);
 
@@ -538,9 +548,19 @@ public class SMIMESignedGenerator
             MimeBodyPart bodyPart)
             throws IOException, MessagingException
         {
-            if (bodyPart.getContent() instanceof Multipart)
+            if (SMIMEUtil.isMultipartContent(bodyPart))
             {
-                Multipart mp = (Multipart)bodyPart.getContent();
+                Object content = bodyPart.getContent();
+                Multipart mp;
+                if (content instanceof Multipart)
+                {
+                    mp = (Multipart)content;
+                }
+                else
+                {
+                    mp = new MimeMultipart(bodyPart.getDataHandler().getDataSource());
+                }
+
                 ContentType contentType = new ContentType(mp.getContentType());
                 String boundary = "--" + contentType.getParameter("boundary");
 
@@ -562,7 +582,7 @@ public class SMIMESignedGenerator
                     writeBodyPart(out, (MimeBodyPart)mp.getBodyPart(i));
                     lOut.writeln();       // CRLF terminator
                 }
-                
+
                 lOut.writeln(boundary + "--");
             }
             else
@@ -593,7 +613,12 @@ public class SMIMESignedGenerator
                     }
                     else
                     {
-                        content.getDataHandler().setCommandMap(addCommands(CommandMap.getDefaultCommandMap()));
+                        CommandMap commandMap = CommandMap.getDefaultCommandMap();
+
+                        if (commandMap instanceof MailcapCommandMap)
+                        {
+                            content.getDataHandler().setCommandMap(addCommands((MailcapCommandMap)commandMap));
+                        }
 
                         content.writeTo(signingStream);
                     }
