@@ -4,7 +4,6 @@ import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.security.spec.AlgorithmParameterSpec;
 import java.security.spec.InvalidParameterSpecException;
-import java.security.spec.PSSParameterSpec;
 
 import org.bouncycastle.asn1.ASN1Integer;
 import org.bouncycastle.asn1.DEROutputStream;
@@ -50,7 +49,7 @@ public abstract class AlgorithmParametersSpi
         protected byte[] engineGetEncoded(
             String format)
         {
-            if (isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
+            if (this.isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
             {
                 return engineGetEncoded();
             }
@@ -117,7 +116,7 @@ public abstract class AlgorithmParametersSpi
     public static class PSS
         extends AlgorithmParametersSpi
     {  
-        PSSParameterSpec currentSpec;
+        PSSParamSpec currentSpec;
     
         /**
          * Return the PKCS#1 ASN.1 structure RSASSA-PSS-params.
@@ -127,7 +126,7 @@ public abstract class AlgorithmParametersSpi
         {
             ByteArrayOutputStream   bOut = new ByteArrayOutputStream();
             DEROutputStream         dOut = new DEROutputStream(bOut);
-            PSSParameterSpec    pssSpec = (PSSParameterSpec)currentSpec;
+            PSSParamSpec    pssSpec = (PSSParamSpec)currentSpec;
             RSASSAPSSparams     pssP = new RSASSAPSSparams(RSASSAPSSparams.DEFAULT_HASH_ALGORITHM, RSASSAPSSparams.DEFAULT_MASK_GEN_FUNCTION, new ASN1Integer(pssSpec.getSaltLength()), RSASSAPSSparams.DEFAULT_TRAILER_FIELD);
 
             dOut.writeObject(pssP);
@@ -153,7 +152,7 @@ public abstract class AlgorithmParametersSpi
             Class paramSpec)
             throws InvalidParameterSpecException
         {
-            if (paramSpec == PSSParameterSpec.class && currentSpec != null)
+            if (paramSpec == PSSParamSpec.class && currentSpec != null)
             {
                 return currentSpec;
             }
@@ -165,12 +164,12 @@ public abstract class AlgorithmParametersSpi
             AlgorithmParameterSpec paramSpec)
             throws InvalidParameterSpecException
         {
-            if (!(paramSpec instanceof PSSParameterSpec))
+            if (!(paramSpec instanceof PSSParamSpec))
             {
                 throw new InvalidParameterSpecException("PSSParameterSpec required to initialise an PSS algorithm parameters object");
             }
     
-            this.currentSpec = (PSSParameterSpec)paramSpec;
+            this.currentSpec = (PSSParamSpec)paramSpec;
         }
     
         protected void engineInit(
@@ -201,7 +200,7 @@ public abstract class AlgorithmParametersSpi
             String format)
             throws IOException
         {
-            if (isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
+            if (this.isASN1FormatString(format) || format.equalsIgnoreCase("X.509"))
             {
                 engineInit(params);
             }
