@@ -137,6 +137,7 @@ public class BCECPrivateKey
     BCECPrivateKey(
         PrivateKeyInfo      info,
         ProviderConfiguration configuration)
+        throws IOException
     {
         this.configuration = configuration;
 
@@ -147,6 +148,7 @@ public class BCECPrivateKey
         String              algorithm,
         PrivateKeyInfo      info,
         ProviderConfiguration configuration)
+        throws IOException
     {
         this.configuration = configuration;
         populateFromPrivKeyInfo(info);
@@ -154,8 +156,9 @@ public class BCECPrivateKey
     }
 
     private void populateFromPrivKeyInfo(PrivateKeyInfo info)
+        throws IOException
     {
-        X962Parameters      params = X962Parameters.getInstance(info.getAlgorithmId().getParameters());
+        X962Parameters      params = X962Parameters.getInstance(info.getPrivateKeyAlgorithm().getParameters());
 
         if (params.isNamedCurve())
         {
@@ -184,15 +187,15 @@ public class BCECPrivateKey
                                             ecP.getSeed());
         }
 
-        if (info.getPrivateKey() instanceof ASN1Integer)
+        if (info.parsePrivateKey() instanceof ASN1Integer)
         {
-            ASN1Integer          derD = ASN1Integer.getInstance(info.getPrivateKey());
+            ASN1Integer          derD = ASN1Integer.getInstance(info.parsePrivateKey());
 
             this.d = derD.getValue();
         }
         else
         {
-            ECPrivateKeyStructure   ec = new ECPrivateKeyStructure((ASN1Sequence)info.getPrivateKey());
+            ECPrivateKeyStructure   ec = new ECPrivateKeyStructure(ASN1Sequence.getInstance(info.parsePrivateKey()));
 
             this.d = ec.getKey();
             this.publicKey = ec.getPublicKey();
