@@ -16,6 +16,7 @@ import org.bouncycastle.crypto.Digest;
 import org.bouncycastle.crypto.digests.GOST3411Digest;
 import org.bouncycastle.crypto.params.ParametersWithRandom;
 import org.bouncycastle.crypto.signers.ECGOST3410Signer;
+import org.bouncycastle.crypto.params.AsymmetricKeyParameter;
 import org.bouncycastle.jcajce.provider.asymmetric.util.ECUtil;
 import org.bouncycastle.jce.interfaces.ECKey;
 import org.bouncycastle.jce.interfaces.ECPublicKey;
@@ -45,7 +46,7 @@ public class SignatureSpi
 
         if (publicKey instanceof ECPublicKey)
         {
-            param = ECUtil.generatePublicKeyParameter(publicKey);
+            param = generatePublicKeyParameter(publicKey);
         }
         else if (publicKey instanceof GOST3410Key)
         {
@@ -209,4 +210,12 @@ public class SignatureSpi
     {
         throw new UnsupportedOperationException("engineSetParameter unsupported");
     }
+
+
+    static AsymmetricKeyParameter generatePublicKeyParameter(
+            PublicKey key)
+    throws InvalidKeyException
+    {
+        return (key instanceof BCECGOST3410PublicKey) ? ((BCECGOST3410PublicKey)key).engineGetKeyParameters() : ECUtil.generatePublicKeyParameter(key);
+    }
 }
