@@ -84,7 +84,10 @@ public class RegressionTest
         new BCFKSStoreTest(),
         new DSTU7624Test(),
         new GOST3412Test(),
-        new GOST3410KeyPairTest()
+        new GOST3410KeyPairTest(),
+        new EdECTest(),
+        new OpenSSHSpecTests(),
+        new SM2CipherTest()
     };
 
     public static void main(
