@@ -1,11 +1,15 @@
 package org.bouncycastle.jsse;
 
+import javax.net.SocketFactory;
+
 /**
  * A BCJSSE-specific interface to expose extended functionality on {@link javax.net.ssl.SSLSocket}
  * implementations.
  */
 public interface BCSSLSocket
 {
+    BCExtendedSSLSession getBCHandshakeSession();
+
     /**
      * Returns an accessor for extended SSL connection data. This method will initiate the initial
      * handshake if necessary and then block until the handshake has been established. If an error
@@ -21,6 +25,19 @@ public interface BCSSLSocket
      */
     BCSSLParameters getParameters();
 
+    /**
+     * Allows explicit setting of the 'host' {@link String} when the {@link SocketFactory} methods
+     * that include it as an argument are not used.
+     * <p>
+     * Must be called prior to attempting to connect the socket to have any effect.
+     * </p>
+     *
+     * @param host
+     *            the server host name with which to connect, or <code>null</code> for the loopback
+     *            address.
+     */
+    void setHost(String host);
+
     /**
      * Sets parameters according to the properties in a {@link BCSSLParameters}.
      * <p>
