@@ -1,41 +1,36 @@
 package org.bouncycastle.jsse.provider;
 
-import javax.net.ssl.X509KeyManager;
-import javax.net.ssl.X509TrustManager;
+import javax.net.ssl.X509ExtendedKeyManager;
 
+import org.bouncycastle.jsse.BCX509ExtendedTrustManager;
 import org.bouncycastle.tls.crypto.TlsCrypto;
 
 final class ContextData
 {
     private final TlsCrypto crypto;
-    private final X509KeyManager km;
-    private final X509TrustManager tm;
+    private final X509ExtendedKeyManager x509KeyManager;
+    private final BCX509ExtendedTrustManager x509TrustManager;
     private final ProvSSLSessionContext clientSessionContext;
     private final ProvSSLSessionContext serverSessionContext;
 
-    ContextData(TlsCrypto crypto, X509KeyManager km, X509TrustManager tm, ProvSSLSessionContext clientSessionContext,
-        ProvSSLSessionContext serverSessionContext)
+    ContextData(TlsCrypto crypto, X509ExtendedKeyManager x509KeyManager, BCX509ExtendedTrustManager x509TrustManager,
+        ProvSSLSessionContext clientSessionContext, ProvSSLSessionContext serverSessionContext)
     {
         this.crypto = crypto;
-        this.km = km;
-        this.tm = tm;
+        this.x509KeyManager = x509KeyManager;
+        this.x509TrustManager = x509TrustManager;
         this.clientSessionContext = clientSessionContext;
         this.serverSessionContext = serverSessionContext;
     }
-
-    ProvSSLSessionContext getClientSessionContext()
-    {
-        return clientSessionContext;
-    }
     
     TlsCrypto getCrypto()
     {
         return crypto;
     }
 
-    X509KeyManager getKeyManager()
+    ProvSSLSessionContext getClientSessionContext()
     {
-        return km;
+        return clientSessionContext;
     }
 
     ProvSSLSessionContext getServerSessionContext()
@@ -43,8 +38,13 @@ final class ContextData
         return serverSessionContext;
     }
 
-    X509TrustManager getTrustManager()
+    X509ExtendedKeyManager getX509KeyManager()
     {
-        return tm;
+        return x509KeyManager;
+    }
+
+    BCX509ExtendedTrustManager getX509TrustManager()
+    {
+        return x509TrustManager;
     }
 }
