@@ -8,6 +8,7 @@ import java.util.List;
 import javax.net.ssl.SNIHostName;
 import javax.net.ssl.SNIMatcher;
 import javax.net.ssl.SNIServerName;
+import javax.net.ssl.StandardConstants;
 
 import org.bouncycastle.jsse.BCSNIHostName;
 import org.bouncycastle.jsse.BCSNIMatcher;
@@ -15,7 +16,7 @@ import org.bouncycastle.jsse.BCSNIServerName;
 import org.bouncycastle.jsse.BCStandardConstants;
 
 abstract class JsseUtils_8
-    extends JsseUtils
+    extends JsseUtils_7
 {
     static class ExportSNIMatcher extends SNIMatcher
     {
@@ -63,9 +64,17 @@ abstract class JsseUtils_8
         }
     }
 
+    static class UnknownServerName extends SNIServerName
+    {
+        UnknownServerName(int type, byte[] encoded)
+        {
+            super(type, encoded);
+        }
+    }
+
     static SNIMatcher exportSNIMatcher(BCSNIMatcher matcher)
     {
-        if (matcher == null)
+        if (null == matcher)
         {
             return null;
         }
@@ -83,37 +92,36 @@ abstract class JsseUtils_8
      */
     static Object exportSNIMatchers(Collection<BCSNIMatcher> matchers)
     {
-        if (matchers == null)
+        if (matchers.isEmpty())
         {
-            return null;
+            return Collections.<SNIMatcher>emptyList();
         }
 
         ArrayList<SNIMatcher> result = new ArrayList<SNIMatcher>(matchers.size());
         for (BCSNIMatcher matcher : matchers)
         {
-            SNIMatcher exported = exportSNIMatcher(matcher);
-            if (exported != null)
-            {
-                result.add(exported);
-            }
+            result.add(exportSNIMatcher(matcher));
         }
-
-        if (!result.isEmpty())
-        {
-            return Collections.unmodifiableList(result);
-        }
-
-        return Collections.<SNIMatcher>emptyList();
+        return Collections.unmodifiableList(result);
     }
 
-    static SNIHostName exportSNIServerName(BCSNIServerName serverName)
+    static SNIServerName exportSNIServerName(BCSNIServerName serverName)
     {
-        if (serverName == null || serverName.getType() != BCStandardConstants.SNI_HOST_NAME)
+        if (null == serverName)
         {
             return null;
         }
 
-        return new SNIHostName(serverName.getEncoded());
+        int type = serverName.getType();
+        byte[] encoded = serverName.getEncoded();
+
+        switch (type)
+        {
+        case BCStandardConstants.SNI_HOST_NAME:
+            return new SNIHostName(encoded);
+        default:
+            return new UnknownServerName(type, encoded);
+        }
     }
 
     /*
@@ -121,32 +129,22 @@ abstract class JsseUtils_8
      */
     static Object exportSNIServerNames(Collection<BCSNIServerName> serverNames)
     {
-        if (serverNames == null)
+        if (serverNames.isEmpty())
         {
-            return null;
+            return Collections.<SNIServerName>emptyList();
         }
 
         ArrayList<SNIServerName> result = new ArrayList<SNIServerName>(serverNames.size());
         for (BCSNIServerName serverName : serverNames)
         {
-            SNIHostName exported = exportSNIServerName(serverName);
-            if (exported != null)
-            {
-                result.add(exported);
-            }
+            result.add(exportSNIServerName(serverName));
         }
-
-        if (!result.isEmpty())
-        {
-            return Collections.unmodifiableList(result);
-        }
-
-        return Collections.<SNIServerName>emptyList();
+        return Collections.unmodifiableList(result);
     }
 
     static BCSNIMatcher importSNIMatcher(SNIMatcher matcher)
     {
-        if (matcher == null)
+        if (null == matcher)
         {
             return null;
         }
@@ -159,41 +157,44 @@ abstract class JsseUtils_8
         return new ImportSNIMatcher(matcher);
     }
 
+    /*
+     * NOTE: Currently argument is Object type to isolate callers from JDK8 type
+     */
     static List<BCSNIMatcher> importSNIMatchers(Object getSNIMatchersResult)
     {
-        if (getSNIMatchersResult == null)
-        {
-            return null;
-        }
-
+        @SuppressWarnings("unchecked")
         Collection<SNIMatcher> matchers = (Collection<SNIMatcher>)getSNIMatchersResult;
 
+        if (matchers.isEmpty())
+        {
+            return Collections.emptyList();
+        }
+
         ArrayList<BCSNIMatcher> result = new ArrayList<BCSNIMatcher>(matchers.size());
         for (SNIMatcher matcher : matchers)
         {
-            BCSNIMatcher imported = importSNIMatcher(matcher);
-            if (imported != null)
-            {
-                result.add(imported);
-            }
+            result.add(importSNIMatcher(matcher));
         }
-
-        if (!result.isEmpty())
-        {
-            return Collections.unmodifiableList(result);
-        }
-
-        return Collections.<BCSNIMatcher>emptyList();
+        return Collections.unmodifiableList(result);
     }
 
-    static BCSNIHostName importSNIServerName(SNIServerName serverName)
+    static BCSNIServerName importSNIServerName(SNIServerName serverName)
     {
-        if (serverName == null || serverName.getType() != BCStandardConstants.SNI_HOST_NAME)
+        if (null == serverName)
         {
             return null;
         }
 
-        return new BCSNIHostName(serverName.getEncoded());
+        int type = serverName.getType();
+        byte[] encoded = serverName.getEncoded();
+
+        switch (type)
+        {
+        case StandardConstants.SNI_HOST_NAME:
+            return new BCSNIHostName(encoded);
+        default:
+            return new BCUnknownServerName(type, encoded);
+        }
     }
 
     /*
@@ -201,28 +202,19 @@ abstract class JsseUtils_8
      */
     static List<BCSNIServerName> importSNIServerNames(Object getServerNamesResult)
     {
-        if (getServerNamesResult == null)
-        {
-            return null;
-        }
-
+        @SuppressWarnings("unchecked")
         Collection<SNIServerName> serverNames = (Collection<SNIServerName>)getServerNamesResult;
 
+        if (serverNames.isEmpty())
+        {
+            return Collections.emptyList();
+        }
+
         ArrayList<BCSNIServerName> result = new ArrayList<BCSNIServerName>(serverNames.size());
         for (SNIServerName serverName : serverNames)
         {
-            BCSNIHostName imported = importSNIServerName(serverName);
-            if (imported != null)
-            {
-                result.add(imported);
-            }
+            result.add(importSNIServerName(serverName));
         }
-
-        if (!result.isEmpty())
-        {
-            return Collections.unmodifiableList(result);
-        }
-
-        return Collections.<BCSNIServerName>emptyList();
+        return Collections.unmodifiableList(result);
     }
 }
