@@ -13,12 +13,16 @@ import java.util.Set;
 import javax.net.ssl.HandshakeCompletedListener;
 import javax.net.ssl.SSLSocket;
 
+import org.bouncycastle.jsse.BCSSLConnection;
 import org.bouncycastle.jsse.BCSSLSocket;
 
 abstract class ProvSSLSocketBase
     extends SSLSocket
     implements BCSSLSocket
 {
+    protected static final boolean provJdkTlsTrustNameService = PropertyUtils
+        .getBooleanSystemProperty("jdk.tls.trustNameService", false);
+
     protected final Closeable socketCloser = new Closeable()
     {
         public void close() throws IOException
@@ -73,6 +77,14 @@ abstract class ProvSSLSocketBase
         super.close();
     }
 
+    // @Override from JDK 9
+    public String getApplicationProtocol()
+    {
+        BCSSLConnection connection = getConnection();
+
+        return connection == null ? null : connection.getApplicationProtocol();
+    }
+
     @Override
     public SocketChannel getChannel()
     {
