@@ -1,6 +1,5 @@
 package org.bouncycastle.tls;
 
-import java.io.ByteArrayOutputStream;
 import java.io.EOFException;
 import java.io.IOException;
 import java.io.InputStream;
@@ -22,10 +21,9 @@ class RecordStream
     private TlsProtocol handler;
     private InputStream input;
     private OutputStream output;
-    private TlsCompression pendingCompression = null, readCompression = null, writeCompression = null;
+    private TlsContext context = null;
     private TlsCipher pendingCipher = null, readCipher = null, writeCipher = null;
     private SequenceNumber readSeqNo = new SequenceNumber(), writeSeqNo = new SequenceNumber();
-    private ByteArrayOutputStream buffer = new ByteArrayOutputStream();
 
     private TlsHandshakeHash handshakeHash = null;
     private SimpleOutputStream handshakeHashUpdater = new SimpleOutputStream()
@@ -36,22 +34,20 @@ class RecordStream
         }
     };
 
-    private ProtocolVersion readVersion = null, writeVersion = null;
-    private boolean restrictReadVersion = true;
+    private ProtocolVersion writeVersion = null;
 
-    private int plaintextLimit, compressedLimit, ciphertextLimit;
+    private int plaintextLimit, ciphertextLimit;
 
     RecordStream(TlsProtocol handler, InputStream input, OutputStream output)
     {
         this.handler = handler;
         this.input = input;
         this.output = output;
-        this.readCompression = new TlsNullCompression();
-        this.writeCompression = this.readCompression;
     }
 
     void init(TlsContext context)
     {
+        this.context = context;
         this.readCipher = new TlsNullNullCipher();
         this.writeCipher = this.readCipher;
         this.handshakeHash = new DeferredHash(context);
@@ -67,18 +63,7 @@ class RecordStream
     void setPlaintextLimit(int plaintextLimit)
     {
         this.plaintextLimit = plaintextLimit;
-        this.compressedLimit = this.plaintextLimit + 1024;
-        this.ciphertextLimit = this.compressedLimit + 1024;
-    }
-
-    ProtocolVersion getReadVersion()
-    {
-        return readVersion;
-    }
-
-    void setReadVersion(ProtocolVersion readVersion)
-    {
-        this.readVersion = readVersion;
+        this.ciphertextLimit = this.plaintextLimit + 1024;
     }
 
     void setWriteVersion(ProtocolVersion writeVersion)
@@ -86,32 +71,18 @@ class RecordStream
         this.writeVersion = writeVersion;
     }
 
-    /**
-     * RFC 5246 E.1. "Earlier versions of the TLS specification were not fully clear on what the
-     * record layer version number (TLSPlaintext.version) should contain when sending ClientHello
-     * (i.e., before it is known which version of the protocol will be employed). Thus, TLS servers
-     * compliant with this specification MUST accept any value {03,XX} as the record layer version
-     * number for ClientHello."
-     */
-    void setRestrictReadVersion(boolean enabled)
+    void setPendingConnectionState(TlsCipher tlsCipher)
     {
-        this.restrictReadVersion = enabled;
-    }
-
-    void setPendingConnectionState(TlsCompression tlsCompression, TlsCipher tlsCipher)
-    {
-        this.pendingCompression = tlsCompression;
         this.pendingCipher = tlsCipher;
     }
 
     void sentWriteCipherSpec()
         throws IOException
     {
-        if (pendingCompression == null || pendingCipher == null)
+        if (pendingCipher == null)
         {
             throw new TlsFatalAlert(AlertDescription.handshake_failure);
         }
-        this.writeCompression = this.pendingCompression;
         this.writeCipher = this.pendingCipher;
         this.writeSeqNo = new SequenceNumber();
     }
@@ -119,11 +90,10 @@ class RecordStream
     void receivedReadCipherSpec()
         throws IOException
     {
-        if (pendingCompression == null || pendingCipher == null)
+        if (pendingCipher == null)
         {
             throw new TlsFatalAlert(AlertDescription.handshake_failure);
         }
-        this.readCompression = this.pendingCompression;
         this.readCipher = this.pendingCipher;
         this.readSeqNo = new SequenceNumber();
     }
@@ -131,13 +101,12 @@ class RecordStream
     void finaliseHandshake()
         throws IOException
     {
-        if (readCompression != pendingCompression || writeCompression != pendingCompression
-            || readCipher != pendingCipher || writeCipher != pendingCipher)
+        if (readCipher != pendingCipher || writeCipher != pendingCipher)
         {
             throw new TlsFatalAlert(AlertDescription.handshake_failure);
         }
-        this.pendingCompression = null;
         this.pendingCipher = null;
+        this.handshakeHash = new DeferredHash(context);
     }
 
     RecordPreview previewRecordHeader(byte[] recordHeader, boolean appDataReady) throws IOException
@@ -155,26 +124,9 @@ class RecordStream
          */
         checkType(type, AlertDescription.unexpected_message);
 
-        if (!restrictReadVersion)
-        {
-            int version = TlsUtils.readVersionRaw(recordHeader, RecordFormat.VERSION_OFFSET);
-            if ((version & 0xffffff00) != 0x0300)
-            {
-                throw new TlsFatalAlert(AlertDescription.illegal_parameter);
-            }
-        }
-        else
-        {
-            ProtocolVersion version = TlsUtils.readVersion(recordHeader, RecordFormat.VERSION_OFFSET);
-            if (readVersion == null)
-            {
-                // Will be set later in 'readRecord'
-            }
-            else if (!version.equals(readVersion))
-            {
-                throw new TlsFatalAlert(AlertDescription.illegal_parameter);
-            }
-        }
+        /*
+         * legacy_record_version (2 octets at RecordFormat.VERSION_OFFSET) is ignored.
+         */
 
         int length = TlsUtils.readUint16(recordHeader, RecordFormat.LENGTH_OFFSET);
 
@@ -185,12 +137,7 @@ class RecordStream
 
         if (type == ContentType.application_data)
         {
-            applicationDataLimit = getPlaintextLimit();
-
-            if (readCompression.getClass() == TlsNullCompression.class)
-            {
-                applicationDataLimit = Math.min(applicationDataLimit, readCipher.getPlaintextLimit(length));
-            }
+            applicationDataLimit = Math.min(getPlaintextLimit(), readCipher.getPlaintextLimit(length));
         }
 
         return new RecordPreview(recordSize, applicationDataLimit);
@@ -200,13 +147,7 @@ class RecordStream
     {
         int applicationDataLimit = Math.max(0, Math.min(getPlaintextLimit(), applicationDataSize));
 
-        int recordSize = applicationDataLimit;
-        if (writeCompression.getClass() != TlsNullCompression.class)
-        {
-            recordSize += 1024;
-        }
-
-        recordSize = writeCipher.getCiphertextLimit(recordSize) + RecordFormat.FRAGMENT_OFFSET;
+        int recordSize = writeCipher.getCiphertextLimit(applicationDataLimit) + RecordFormat.FRAGMENT_OFFSET;
 
         return new RecordPreview(recordSize, applicationDataLimit);
     }
@@ -233,26 +174,9 @@ class RecordStream
          */
         checkType(type, AlertDescription.unexpected_message);
 
-        if (!restrictReadVersion)
-        {
-            int version = TlsUtils.readVersionRaw(input, inputOff + RecordFormat.VERSION_OFFSET);
-            if ((version & 0xffffff00) != 0x0300)
-            {
-                throw new TlsFatalAlert(AlertDescription.illegal_parameter);
-            }
-        }
-        else
-        {
-            ProtocolVersion version = TlsUtils.readVersion(input, inputOff + RecordFormat.VERSION_OFFSET);
-            if (readVersion == null)
-            {
-                readVersion = version;
-            }
-            else if (!version.equals(readVersion))
-            {
-                throw new TlsFatalAlert(AlertDescription.illegal_parameter);
-            }
-        }
+        /*
+         * legacy_record_version (2 octets at RecordFormat.VERSION_OFFSET) is ignored.
+         */
 
         checkLength(length, ciphertextLimit, AlertDescription.record_overflow);
 
@@ -277,26 +201,9 @@ class RecordStream
          */
         checkType(type, AlertDescription.unexpected_message);
 
-        if (!restrictReadVersion)
-        {
-            int version = TlsUtils.readVersionRaw(inputRecord.buf, RecordFormat.VERSION_OFFSET);
-            if ((version & 0xffffff00) != 0x0300)
-            {
-                throw new TlsFatalAlert(AlertDescription.illegal_parameter);
-            }
-        }
-        else
-        {
-            ProtocolVersion version = TlsUtils.readVersion(inputRecord.buf, RecordFormat.VERSION_OFFSET);
-            if (readVersion == null)
-            {
-                readVersion = version;
-            }
-            else if (!version.equals(readVersion))
-            {
-                throw new TlsFatalAlert(AlertDescription.illegal_parameter);
-            }
-        }
+        /*
+         * legacy_record_version (2 octets at RecordFormat.VERSION_OFFSET) is ignored.
+         */
 
         int length = TlsUtils.readUint16(inputRecord.buf, RecordFormat.LENGTH_OFFSET);
 
@@ -324,26 +231,7 @@ class RecordStream
         long seqNo = readSeqNo.nextValue(AlertDescription.unexpected_message);
         byte[] decoded = readCipher.decodeCiphertext(seqNo, type, ciphertext, off, len);
 
-        checkLength(decoded.length, compressedLimit, AlertDescription.record_overflow);
-
-        /*
-         * TODO RFC 5246 6.2.2. Implementation note: Decompression functions are responsible for
-         * ensuring that messages cannot cause internal buffer overflows.
-         */
-        OutputStream cOut = readCompression.decompress(buffer);
-        if (cOut != buffer)
-        {
-            cOut.write(decoded, 0, decoded.length);
-            cOut.flush();
-            decoded = getBufferContents();
-        }
-
-        /*
-         * RFC 5246 6.2.2. If the decompression function encounters a TLSCompressed.fragment that
-         * would decompress to a length in excess of 2^14 bytes, it should report a fatal
-         * decompression failure error.
-         */
-        checkLength(decoded.length, plaintextLimit, AlertDescription.decompression_failure);
+        checkLength(decoded.length, plaintextLimit, AlertDescription.record_overflow);
 
         /*
          * RFC 5246 6.2.1 Implementations MUST NOT send zero-length fragments of Handshake, Alert,
@@ -386,29 +274,9 @@ class RecordStream
             throw new TlsFatalAlert(AlertDescription.internal_error);
         }
 
-        OutputStream cOut = writeCompression.compress(buffer);
-
         long seqNo = writeSeqNo.nextValue(AlertDescription.internal_error);
 
-        byte[] ciphertext;
-        if (cOut == buffer)
-        {
-            ciphertext = writeCipher.encodePlaintext(seqNo, type, plaintext, plaintextOffset, plaintextLength);
-        }
-        else
-        {
-            cOut.write(plaintext, plaintextOffset, plaintextLength);
-            cOut.flush();
-            byte[] compressed = getBufferContents();
-
-            /*
-             * RFC 5246 6.2.2. Compression must be lossless and may not increase the content length
-             * by more than 1024 bytes.
-             */
-            checkLength(compressed.length, plaintextLength + 1024, AlertDescription.internal_error);
-
-            ciphertext = writeCipher.encodePlaintext(seqNo, type, compressed, 0, compressed.length);
-        }
+        byte[] ciphertext = writeCipher.encodePlaintext(seqNo, type, plaintext, plaintextOffset, plaintextLength);
 
         /*
          * RFC 5246 6.2.3. The length may not exceed 2^14 + 2048.
@@ -498,13 +366,6 @@ class RecordStream
         output.flush();
     }
 
-    private byte[] getBufferContents()
-    {
-        byte[] contents = buffer.toByteArray();
-        buffer.reset();
-        return contents;
-    }
-
     private static void checkType(short type, short alertDescription)
         throws IOException
     {
