@@ -71,24 +71,16 @@ public abstract class TlsProtocol
     private volatile boolean resumableHandshake = false;
     private volatile int appDataSplitMode = ADS_MODE_1_Nsub1;
 
-    // TODO[tls-ops] Investigate whether we can handle (expected/actual) verify data using TlsSecret
-    private byte[] expected_verify_data = null;
-
     protected TlsSession tlsSession = null;
     protected SessionParameters sessionParameters = null;
-    protected SecurityParameters securityParameters = null;
-    protected Certificate localCertificate = null;
-    protected Certificate peerCertificate = null;
 
     protected int[] offeredCipherSuites = null;
-    protected short[] offeredCompressionMethods = null;
     protected Hashtable clientExtensions = null;
     protected Hashtable serverExtensions = null;
 
     protected short connection_state = CS_START;
     protected boolean resumedSession = false;
     protected boolean receivedChangeCipherSpec = false;
-    protected boolean secure_renegotiation = false;
     protected boolean allowCertificateStatus = false;
     protected boolean expectSessionTicket = false;
 
@@ -110,6 +102,20 @@ public abstract class TlsProtocol
         this.recordStream = new RecordStream(this, input, output);
     }
 
+//    public boolean renegotiate() throws IOException
+//    {
+//        TlsContext context = getContext();
+//        if (null == context.getSecurityParametersConnection() || isClosed())
+//        {
+//            throw new IllegalStateException("No connection");
+//        }
+//        if (!appDataReady)
+//        {
+//            throw new IllegalStateException("Initial handshake in progress");
+//        }
+//        return null == context.getSecurityParametersHandshake() && CS_END == connection_state;
+//    }
+
     public void resumeHandshake() throws IOException
     {
         if (!blocking)
@@ -135,6 +141,16 @@ public abstract class TlsProtocol
 
     protected abstract TlsPeer getPeer();
 
+    protected int getRenegotiationPolicy()
+    {
+//        SecurityParameters sp = getContext().getSecurityParametersConnection();
+//        if (null != sp && sp.isSecureRenegotiation())
+//        {
+//            return getPeer().getRenegotiationPolicy();
+//        }
+        return RenegotiationPolicy.DENY;
+    }
+
     protected void handleAlertMessage(short alertLevel, short alertDescription)
         throws IOException
     {
@@ -155,17 +171,30 @@ public abstract class TlsProtocol
     protected void handleAlertWarningMessage(short alertDescription)
         throws IOException
     {
+        switch (alertDescription)
+        {
         /*
          * RFC 5246 7.2.1. The other party MUST respond with a close_notify alert of its own
          * and close down the connection immediately, discarding any pending writes.
          */
-        if (alertDescription == AlertDescription.close_notify)
+        case AlertDescription.close_notify:
         {
             if (!appDataReady)
             {
                 throw new TlsFatalAlert(AlertDescription.handshake_failure);
             }
             handleClose(false);
+            break;
+        }
+        case AlertDescription.no_certificate:
+        {
+            throw new TlsFatalAlert(AlertDescription.unexpected_message);
+        }
+        case AlertDescription.no_renegotiation:
+        {
+            // TODO[reneg] Give peer the option to tolerate this
+            throw new TlsFatalAlert(AlertDescription.handshake_failure);
+        }
         }
     }
 
@@ -187,7 +216,7 @@ public abstract class TlsProtocol
 
             raiseAlertWarning(AlertDescription.close_notify, "Connection closed");
 
-            if (!appDataReady)
+            if (connection_state != CS_END)
             {
                 cleanupHandshake();
             }
@@ -224,7 +253,7 @@ public abstract class TlsProtocol
         // TODO This isn't quite in the right place. Also, as of TLS 1.1 the above is obsolete.
         invalidateSession();
 
-        if (!appDataReady)
+        if (connection_state != CS_END)
         {
             cleanupHandshake();
         }
@@ -235,10 +264,32 @@ public abstract class TlsProtocol
     protected abstract void handleHandshakeMessage(short type, ByteArrayInputStream buf)
         throws IOException;
 
+    protected boolean handleRenegotiation() throws IOException
+    {
+        switch (getRenegotiationPolicy())
+        {
+        case RenegotiationPolicy.ACCEPT:
+        {
+            beginHandshake(true);
+            return true;
+        }
+        case RenegotiationPolicy.IGNORE:
+        {
+            return false;
+        }
+        case RenegotiationPolicy.DENY:
+        default:
+        {
+            refuseRenegotiation();
+            return false;
+        }
+        }
+    }
+
     protected void applyMaxFragmentLengthExtension()
         throws IOException
     {
-        short maxFragmentLength = securityParameters.getMaxFragmentLength();
+        short maxFragmentLength = getContext().getSecurityParametersHandshake().getMaxFragmentLength();
         if (maxFragmentLength >= 0)
         {
             if (!MaxFragmentLength.isValid(maxFragmentLength))
@@ -274,28 +325,42 @@ public abstract class TlsProtocol
         }
     }
 
+    protected void beginHandshake(boolean renegotiation)
+        throws IOException
+    {
+        this.connection_state = CS_START;
+
+        AbstractTlsContext context = getContextAdmin(); 
+        TlsPeer peer = getPeer();
+
+        context.handshakeBeginning(peer);
+
+        SecurityParameters securityParameters = context.getSecurityParametersHandshake();
+        if (renegotiation != securityParameters.renegotiating)
+        {
+            throw new TlsFatalAlert(AlertDescription.internal_error);
+        }
+
+        securityParameters.extendedPadding = peer.shouldUseExtendedPadding();
+    }
+
     protected void cleanupHandshake()
     {
-        if (this.expected_verify_data != null)
+        SecurityParameters securityParameters = getContext().getSecurityParameters();
+        if (null != securityParameters)
         {
-            Arrays.fill(this.expected_verify_data, (byte)0);
-            this.expected_verify_data = null;
+            securityParameters.clear();
         }
 
         this.tlsSession = null;
         this.sessionParameters = null;
-        this.securityParameters.clear();
-        this.localCertificate = null;
-        this.peerCertificate = null;
 
         this.offeredCipherSuites = null;
-        this.offeredCompressionMethods = null;
         this.clientExtensions = null;
         this.serverExtensions = null;
 
         this.resumedSession = false;
         this.receivedChangeCipherSpec = false;
-        this.secure_renegotiation = false;
         this.allowCertificateStatus = false;
         this.expectSessionTicket = false;
     }
@@ -330,15 +395,17 @@ public abstract class TlsProtocol
 
             if (this.sessionParameters == null)
             {
+                SecurityParameters securityParameters = getContext().getSecurityParametersHandshake();
                 this.sessionParameters = new SessionParameters.Builder()
-                    .setCipherSuite(this.securityParameters.getCipherSuite())
-                    .setCompressionAlgorithm(this.securityParameters.getCompressionAlgorithm())
-                    .setLocalCertificate(this.localCertificate)
-                    .setMasterSecret(getContext().getCrypto().adoptSecret(this.securityParameters.getMasterSecret()))
-                    .setNegotiatedVersion(getContext().getServerVersion())
-                    .setPeerCertificate(this.peerCertificate)
-                    .setPSKIdentity(this.securityParameters.getPSKIdentity())
-                    .setSRPIdentity(this.securityParameters.getSRPIdentity())
+                    .setCipherSuite(securityParameters.getCipherSuite())
+                    .setCompressionAlgorithm(securityParameters.getCompressionAlgorithm())
+                    .setExtendedMasterSecret(securityParameters.isExtendedMasterSecret())
+                    .setLocalCertificate(securityParameters.getLocalCertificate())
+                    .setMasterSecret(getContext().getCrypto().adoptSecret(securityParameters.getMasterSecret()))
+                    .setNegotiatedVersion(securityParameters.getNegotiatedVersion())
+                    .setPeerCertificate(securityParameters.getPeerCertificate())
+                    .setPSKIdentity(securityParameters.getPSKIdentity())
+                    .setSRPIdentity(securityParameters.getSRPIdentity())
                     // TODO Consider filtering extensions that aren't relevant to resumed sessions
                     .setServerExtensions(this.serverExtensions)
                     .build();
@@ -346,9 +413,7 @@ public abstract class TlsProtocol
                 this.tlsSession = TlsUtils.importSession(this.tlsSession.getSessionID(), this.sessionParameters);
             }
 
-            getContextAdmin().setSession(this.tlsSession);
-
-            getPeer().notifyHandshakeComplete();
+            getContextAdmin().handshakeComplete(getPeer(), this.tlsSession);
         }
         finally
         {
@@ -443,31 +508,31 @@ public abstract class TlsProtocol
                 break;
             }
 
-            checkReceivedChangeCipherSpec(connection_state == CS_END || type == HandshakeType.finished);
-
             /*
              * RFC 2246 7.4.9. The value handshake_messages includes all handshake messages
              * starting at client hello up to, but not including, this finished message.
              * [..] Note: [Also,] Hello Request messages are omitted from handshake hashes.
              */
-            switch (type)
+            if (HandshakeType.hello_request != type)
             {
-            case HandshakeType.hello_request:
-                break;
-            case HandshakeType.finished:
-            {
-                TlsContext ctx = getContext();
-                if (this.expected_verify_data == null
-                    && ctx.getSecurityParameters().getMasterSecret() != null)
+                if (HandshakeType.finished == type)
                 {
-                    this.expected_verify_data = createVerifyData(!ctx.isServer());
+                    checkReceivedChangeCipherSpec(true);
+
+                    TlsContext ctx = getContext();
+                    SecurityParameters securityParameters = ctx.getSecurityParametersHandshake();
+
+                    if (securityParameters.getMasterSecret() != null)
+                    {
+                        securityParameters.peerVerifyData = createVerifyData(!ctx.isServer());
+                    }
+                }
+                else
+                {
+                    checkReceivedChangeCipherSpec(false);
                 }
 
-                // NB: Fall through to next case label
-            }
-            default:
                 queue.copyTo(recordStream.getHandshakeHashUpdater(), totalLength);
-                break;
             }
 
             queue.removeData(4);
@@ -1152,11 +1217,23 @@ public abstract class TlsProtocol
     protected void processFinishedMessage(ByteArrayInputStream buf)
         throws IOException
     {
+        TlsContext ctx = getContext();
+        SecurityParameters securityParameters = ctx.getSecurityParametersHandshake();
+
+        byte[] expected_verify_data = securityParameters.getPeerVerifyData();
         if (expected_verify_data == null)
         {
             throw new TlsFatalAlert(AlertDescription.internal_error);
         }
 
+        if (ctx.isServer() ^ resumedSession)
+        {
+            if (!resumedSession || securityParameters.isExtendedMasterSecret())
+            {
+                securityParameters.tlsUnique = expected_verify_data;
+            }
+        }
+
         byte[] verify_data = TlsUtils.readFully(expected_verify_data.length, buf);
 
         assertEmpty(buf);
@@ -1171,11 +1248,6 @@ public abstract class TlsProtocol
              */
             throw new TlsFatalAlert(AlertDescription.decrypt_error);
         }
-
-        if (null == securityParameters.getTLSUnique())
-        {
-            securityParameters.tlsUnique = verify_data;
-        }
     }
 
     protected void raiseAlertFatal(short alertDescription, String message, Throwable cause)
@@ -1205,28 +1277,26 @@ public abstract class TlsProtocol
         safeWriteRecord(ContentType.alert, alert, 0, 2);
     }
 
-    /** @deprecated */
-    protected void sendCertificateMessage(Certificate certificate)
-        throws IOException
-    {
-        sendCertificateMessage(certificate, null);
-    }
-
     protected void sendCertificateMessage(Certificate certificate, OutputStream endPointHash)
         throws IOException
     {
-        if (certificate == null)
+        TlsContext context = getContext();
+        SecurityParameters securityParameters = context.getSecurityParametersHandshake();
+        if (null != securityParameters.getLocalCertificate())
+        {
+            throw new TlsFatalAlert(AlertDescription.internal_error);
+        }
+
+        if (null == certificate)
         {
             certificate = Certificate.EMPTY_CHAIN;
         }
 
         HandshakeMessage message = new HandshakeMessage(HandshakeType.certificate);
-
-        certificate.encode(getContext(), message, endPointHash);
-
+        certificate.encode(context, message, endPointHash);
         message.writeToRecordStream();
 
-        this.localCertificate = certificate;
+        securityParameters.localCertificate = certificate;
     }
 
     protected void sendChangeCipherSpecMessage()
@@ -1240,18 +1310,24 @@ public abstract class TlsProtocol
     protected void sendFinishedMessage()
         throws IOException
     {
-        byte[] verify_data = createVerifyData(getContext().isServer());
+        TlsContext ctx = getContext();
+        SecurityParameters securityParameters = ctx.getSecurityParametersHandshake();
+
+        byte[] verify_data = securityParameters.localVerifyData = createVerifyData(ctx.isServer());
+
+        if (!ctx.isServer() ^ resumedSession)
+        {
+            if (!resumedSession || securityParameters.isExtendedMasterSecret())
+            {
+                securityParameters.tlsUnique = verify_data;
+            }
+        }
 
         HandshakeMessage message = new HandshakeMessage(HandshakeType.finished, verify_data.length);
 
         message.write(verify_data);
 
         message.writeToRecordStream();
-
-        if (null == securityParameters.getTLSUnique())
-        {
-            securityParameters.tlsUnique = verify_data;
-        }
     }
 
     protected void sendSupplementalDataMessage(Vector supplementalData)
@@ -1293,7 +1369,7 @@ public abstract class TlsProtocol
 
     public boolean isHandshaking()
     {
-        return securityParameters != null && this.connection_state != CS_END && !isClosed();
+        return !isClosed() && null != getContext().getSecurityParametersHandshake();
     }
 
     protected short processMaxFragmentLengthExtension(Hashtable clientExtensions, Hashtable serverExtensions,
@@ -1362,7 +1438,7 @@ public abstract class TlsProtocol
 
         try
         {
-            context.getSecurityParameters().masterSecret = TlsUtils.calculateMasterSecret(context, preMasterSecret);
+            context.getSecurityParametersHandshake().masterSecret = TlsUtils.calculateMasterSecret(context, preMasterSecret);
         }
         finally
         {
@@ -1386,23 +1462,27 @@ public abstract class TlsProtocol
 
         assertEmpty(input);
 
-        ByteArrayInputStream buf = new ByteArrayInputStream(extBytes);
-
         // Integer -> byte[]
         Hashtable extensions = new Hashtable();
 
-        while (buf.available() > 0)
+        if (extBytes.length > 0)
         {
-            Integer extension_type = Integers.valueOf(TlsUtils.readUint16(buf));
-            byte[] extension_data = TlsUtils.readOpaque16(buf);
+            ByteArrayInputStream buf = new ByteArrayInputStream(extBytes);
 
-            /*
-             * RFC 3546 2.3 There MUST NOT be more than one extension of the same type.
-             */
-            if (null != extensions.put(extension_type, extension_data))
+            do
             {
-                throw new TlsFatalAlert(AlertDescription.illegal_parameter);
+                Integer extension_type = Integers.valueOf(TlsUtils.readUint16(buf));
+                byte[] extension_data = TlsUtils.readOpaque16(buf);
+
+                /*
+                 * RFC 3546 2.3 There MUST NOT be more than one extension of the same type.
+                 */
+                if (null != extensions.put(extension_type, extension_data))
+                {
+                    throw new TlsFatalAlert(AlertDescription.illegal_parameter);
+                }
             }
+            while (buf.available() > 0);
         }
 
         return extensions;
@@ -1411,7 +1491,7 @@ public abstract class TlsProtocol
     protected static Vector readSupplementalDataMessage(ByteArrayInputStream input)
         throws IOException
     {
-        byte[] supp_data = TlsUtils.readOpaque24(input);
+        byte[] supp_data = TlsUtils.readOpaque24(input, 1);
 
         assertEmpty(input);
 
@@ -1450,6 +1530,11 @@ public abstract class TlsProtocol
     protected static void writeExtensions(OutputStream output, Hashtable extensions)
         throws IOException
     {
+        if (null == extensions || extensions.isEmpty())
+        {
+            return;
+        }
+
         ByteArrayOutputStream buf = new ByteArrayOutputStream();
 
         /*
@@ -1590,9 +1675,9 @@ public abstract class TlsProtocol
         case CipherSuite.TLS_ECDHE_ECDSA_WITH_CAMELLIA_128_CBC_SHA256:
         case CipherSuite.TLS_ECDHE_ECDSA_WITH_CAMELLIA_128_GCM_SHA256:
         case CipherSuite.TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256:
-        case CipherSuite.DRAFT_TLS_ECDHE_PSK_WITH_AES_128_CCM_8_SHA256:
-        case CipherSuite.DRAFT_TLS_ECDHE_PSK_WITH_AES_128_CCM_SHA256:
-        case CipherSuite.DRAFT_TLS_ECDHE_PSK_WITH_AES_128_GCM_SHA256:
+        case CipherSuite.TLS_ECDHE_PSK_WITH_AES_128_CCM_8_SHA256:
+        case CipherSuite.TLS_ECDHE_PSK_WITH_AES_128_CCM_SHA256:
+        case CipherSuite.TLS_ECDHE_PSK_WITH_AES_128_GCM_SHA256:
         case CipherSuite.DRAFT_TLS_ECDHE_PSK_WITH_AES_128_OCB:
         case CipherSuite.DRAFT_TLS_ECDHE_PSK_WITH_AES_256_OCB:
         case CipherSuite.TLS_ECDHE_PSK_WITH_ARIA_128_CBC_SHA256:
@@ -1687,7 +1772,7 @@ public abstract class TlsProtocol
         case CipherSuite.TLS_ECDHE_ECDSA_WITH_ARIA_256_GCM_SHA384:
         case CipherSuite.TLS_ECDHE_ECDSA_WITH_CAMELLIA_256_CBC_SHA384:
         case CipherSuite.TLS_ECDHE_ECDSA_WITH_CAMELLIA_256_GCM_SHA384:
-        case CipherSuite.DRAFT_TLS_ECDHE_PSK_WITH_AES_256_GCM_SHA384:
+        case CipherSuite.TLS_ECDHE_PSK_WITH_AES_256_GCM_SHA384:
         case CipherSuite.TLS_ECDHE_PSK_WITH_ARIA_256_CBC_SHA384:
         case CipherSuite.TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384:
         case CipherSuite.TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384:
