@@ -3,12 +3,10 @@ package org.bouncycastle.tls;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
-import java.util.Vector;
 
 import org.bouncycastle.tls.crypto.TlsCertificate;
 import org.bouncycastle.tls.crypto.TlsCryptoParameters;
 import org.bouncycastle.tls.crypto.TlsSecret;
-import org.bouncycastle.util.io.Streams;
 
 /**
  * (D)TLS RSA key exchange.
@@ -16,13 +14,24 @@ import org.bouncycastle.util.io.Streams;
 public class TlsRSAKeyExchange
     extends AbstractTlsKeyExchange
 {
+    private static int checkKeyExchange(int keyExchange)
+    {
+        switch (keyExchange)
+        {
+        case KeyExchangeAlgorithm.RSA:
+            return keyExchange;
+        default:
+            throw new IllegalArgumentException("unsupported key exchange algorithm");
+        }
+    }
+
     protected TlsCredentialedDecryptor serverCredentials = null;
     protected TlsCertificate serverCertificate;
     protected TlsSecret preMasterSecret;
 
-    public TlsRSAKeyExchange(Vector supportedSignatureAlgorithms)
+    public TlsRSAKeyExchange(int keyExchange)
     {
-        super(KeyExchangeAlgorithm.RSA, supportedSignatureAlgorithms);
+        super(checkKeyExchange(keyExchange));
     }
 
     public void skipServerCredentials()
@@ -34,25 +43,14 @@ public class TlsRSAKeyExchange
     public void processServerCredentials(TlsCredentials serverCredentials)
         throws IOException
     {
-        if (!(serverCredentials instanceof TlsCredentialedDecryptor))
-        {
-            throw new TlsFatalAlert(AlertDescription.internal_error);
-        }
-
-        this.serverCredentials = (TlsCredentialedDecryptor)serverCredentials;
+        this.serverCredentials = TlsUtils.requireDecryptorCredentials(serverCredentials);
     }
 
     public void processServerCertificate(Certificate serverCertificate)
         throws IOException
     {
-        if (serverCertificate.isEmpty())
-        {
-            throw new TlsFatalAlert(AlertDescription.bad_certificate);
-        }
-
-        checkServerCertSigAlg(serverCertificate);
-
-        this.serverCertificate = serverCertificate.getCertificateAt(0).useInRole(ConnectionEnd.server, keyExchange);
+        this.serverCertificate = serverCertificate.getCertificateAt(0).useInRole(ConnectionEnd.server,
+            KeyExchangeAlgorithm.RSA);
     }
 
     public short[] getClientCertificateTypes()
@@ -64,10 +62,7 @@ public class TlsRSAKeyExchange
     public void processClientCredentials(TlsCredentials clientCredentials)
         throws IOException
     {
-        if (!(clientCredentials instanceof TlsCredentialedSigner))
-        {
-            throw new TlsFatalAlert(AlertDescription.internal_error);
-        }
+        TlsUtils.requireSignerCredentials(clientCredentials);
     }
 
     public void generateClientKeyExchange(OutputStream output)
