@@ -4,6 +4,9 @@ import java.io.IOException;
 import java.util.Hashtable;
 import java.util.Vector;
 
+import org.bouncycastle.tls.crypto.TlsDHConfig;
+import org.bouncycastle.tls.crypto.TlsECConfig;
+
 /**
  * Interface describing a TLS server endpoint.
  */
@@ -12,6 +15,16 @@ public interface TlsServer
 {
     void init(TlsServerContext context);
 
+    /**
+     * Return the specified session, if available. Note that the peer's certificate
+     * chain for the session (if any) may need to be periodically revalidated.
+     * 
+     * @param sessionID the ID of the session to resume.
+     * @return A {@link TlsSession} with the specified session ID, or null.
+     * @see SessionParameters#getPeerCertificate()
+     */
+    TlsSession getSessionToResume(byte[] sessionID);
+
     void notifyClientVersion(ProtocolVersion clientVersion) throws IOException;
 
     void notifyFallback(boolean isFallback) throws IOException;
@@ -19,9 +32,6 @@ public interface TlsServer
     void notifyOfferedCipherSuites(int[] offeredCipherSuites)
         throws IOException;
 
-    void notifyOfferedCompressionMethods(short[] offeredCompressionMethods)
-        throws IOException;
-
     // Hashtable is (Integer -> byte[])
     void processClientExtensions(Hashtable clientExtensions)
         throws IOException;
@@ -32,9 +42,6 @@ public interface TlsServer
     int getSelectedCipherSuite()
         throws IOException;
 
-    short getSelectedCompressionMethod()
-        throws IOException;
-
     // Hashtable is (Integer -> byte[])
     Hashtable getServerExtensions()
         throws IOException;
@@ -66,12 +73,17 @@ public interface TlsServer
     CertificateStatus getCertificateStatus()
         throws IOException;
 
-    TlsKeyExchange getKeyExchange()
-        throws IOException;
-
     CertificateRequest getCertificateRequest()
         throws IOException;
 
+    TlsPSKIdentityManager getPSKIdentityManager() throws IOException;
+
+    TlsSRPLoginParameters getSRPLoginParameters() throws IOException;
+
+    TlsDHConfig getDHConfig() throws IOException;
+
+    TlsECConfig getECDHConfig() throws IOException;
+
     // Vector is (SupplementalDataEntry)
     void processClientSupplementalData(Vector clientSupplementalData)
         throws IOException;
