@@ -21,7 +21,9 @@ class TlsSessionImpl implements TlsSession
 
         this.sessionID = Arrays.clone(sessionID);
         this.sessionParameters = sessionParameters;
-        this.resumable = sessionID.length > 0;
+        this.resumable = sessionID.length > 0
+            && null != sessionParameters
+            && sessionParameters.isExtendedMasterSecret();
     }
 
     public synchronized SessionParameters exportSessionParameters()
