@@ -34,7 +34,7 @@ public class TlsImplUtils
 
     public static byte[] calculateKeyBlock(TlsCryptoParameters cryptoParams, int length)
     {
-        SecurityParameters securityParameters = cryptoParams.getSecurityParameters();
+        SecurityParameters securityParameters = cryptoParams.getSecurityParametersHandshake();
         TlsSecret master_secret = securityParameters.getMasterSecret();
         byte[] seed = Arrays.concatenate(securityParameters.getServerRandom(), securityParameters.getClientRandom());
         return PRF(cryptoParams, master_secret, ExporterLabel.key_expansion, seed, length).extract();
@@ -42,7 +42,7 @@ public class TlsImplUtils
 
     public static TlsSecret PRF(TlsCryptoParameters cryptoParams, TlsSecret secret, String asciiLabel, byte[] seed, int length)
     {
-        int prfAlgorithm = cryptoParams.getSecurityParameters().getPrfAlgorithm();
+        int prfAlgorithm = cryptoParams.getSecurityParametersHandshake().getPrfAlgorithm();
 
         return secret.deriveUsingPRF(prfAlgorithm, asciiLabel, seed, length);
     }
