@@ -1,7 +1,7 @@
 package org.bouncycastle.tls.crypto.impl.bc;
 
 import org.bouncycastle.crypto.DSA;
-import org.bouncycastle.crypto.params.AsymmetricKeyParameter;
+import org.bouncycastle.crypto.params.DSAPrivateKeyParameters;
 import org.bouncycastle.crypto.signers.DSASigner;
 import org.bouncycastle.crypto.signers.HMacDSAKCalculator;
 import org.bouncycastle.tls.SignatureAlgorithm;
@@ -12,14 +12,14 @@ import org.bouncycastle.tls.SignatureAlgorithm;
 public class BcTlsDSASigner
     extends BcTlsDSSSigner
 {
-    public BcTlsDSASigner(BcTlsCrypto crypto, AsymmetricKeyParameter privateKey)
+    public BcTlsDSASigner(BcTlsCrypto crypto, DSAPrivateKeyParameters privateKey)
     {
         super(crypto, privateKey);
     }
 
     protected DSA createDSAImpl(short hashAlgorithm)
     {
-        return new DSASigner(new HMacDSAKCalculator(BcTlsCrypto.createDigest(hashAlgorithm)));
+        return new DSASigner(new HMacDSAKCalculator(crypto.createDigest(hashAlgorithm)));
     }
 
     protected short getSignatureAlgorithm()
