@@ -1,7 +1,7 @@
 package org.bouncycastle.tls.crypto.impl.bc;
 
 import org.bouncycastle.crypto.DSA;
-import org.bouncycastle.crypto.params.AsymmetricKeyParameter;
+import org.bouncycastle.crypto.params.ECPrivateKeyParameters;
 import org.bouncycastle.crypto.signers.ECDSASigner;
 import org.bouncycastle.crypto.signers.HMacDSAKCalculator;
 import org.bouncycastle.tls.SignatureAlgorithm;
@@ -12,14 +12,14 @@ import org.bouncycastle.tls.SignatureAlgorithm;
 public class BcTlsECDSASigner
     extends BcTlsDSSSigner
 {
-    public BcTlsECDSASigner(BcTlsCrypto crypto, AsymmetricKeyParameter privateKey)
+    public BcTlsECDSASigner(BcTlsCrypto crypto, ECPrivateKeyParameters privateKey)
     {
         super(crypto, privateKey);
     }
 
     protected DSA createDSAImpl(short hashAlgorithm)
     {
-        return new ECDSASigner(new HMacDSAKCalculator(BcTlsCrypto.createDigest(hashAlgorithm)));
+        return new ECDSASigner(new HMacDSAKCalculator(crypto.createDigest(hashAlgorithm)));
     }
 
     protected short getSignatureAlgorithm()
