@@ -17,13 +17,14 @@ import org.bouncycastle.asn1.ASN1Encoding;
 import org.bouncycastle.asn1.ASN1ObjectIdentifier;
 import org.bouncycastle.asn1.ASN1OctetString;
 import org.bouncycastle.asn1.ASN1Primitive;
+import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
 import org.bouncycastle.asn1.x509.Certificate;
 import org.bouncycastle.asn1.x509.Extensions;
 import org.bouncycastle.asn1.x509.KeyUsage;
+import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
 import org.bouncycastle.asn1.x509.TBSCertificate;
 import org.bouncycastle.jcajce.util.JcaJceHelper;
 import org.bouncycastle.tls.AlertDescription;
-import org.bouncycastle.tls.ClientCertificateType;
 import org.bouncycastle.tls.ConnectionEnd;
 import org.bouncycastle.tls.KeyExchangeAlgorithm;
 import org.bouncycastle.tls.SignatureAlgorithm;
@@ -31,6 +32,7 @@ import org.bouncycastle.tls.TlsFatalAlert;
 import org.bouncycastle.tls.crypto.TlsCertificate;
 import org.bouncycastle.tls.crypto.TlsCryptoException;
 import org.bouncycastle.tls.crypto.TlsVerifier;
+import org.bouncycastle.tls.crypto.impl.RSAUtil;
 
 /**
  * Implementation class for a single X.509 certificate based on the JCA.
@@ -101,83 +103,39 @@ public class JcaTlsCertificate
 
         switch (signatureAlgorithm)
         {
+        case SignatureAlgorithm.rsa:
+            validateRSA_PKCS1();
+            return new JcaTlsRSAVerifier(crypto, getPubKeyRSA());
+
         case SignatureAlgorithm.dsa:
-            return new JcaTlsDSAVerifier(getPubKeyDSS(), crypto.getHelper());
+            return new JcaTlsDSAVerifier(crypto, getPubKeyDSS());
 
         case SignatureAlgorithm.ecdsa:
-            return new JcaTlsECDSAVerifier(getPubKeyEC(), crypto.getHelper());
+            return new JcaTlsECDSAVerifier(crypto, getPubKeyEC());
 
-        case SignatureAlgorithm.rsa:
-            return new JcaTlsRSAVerifier(getPubKeyRSA(), crypto.getHelper());
+        case SignatureAlgorithm.ed25519:
+            return new JcaTlsEd25519Verifier(crypto, getPubKeyEd25519());
+
+        case SignatureAlgorithm.ed448:
+            return new JcaTlsEd448Verifier(crypto, getPubKeyEd448());
+
+        case SignatureAlgorithm.rsa_pss_rsae_sha256:
+        case SignatureAlgorithm.rsa_pss_rsae_sha384:
+        case SignatureAlgorithm.rsa_pss_rsae_sha512:
+            validateRSA_PSS_RSAE();
+            return new JcaTlsRSAPSSVerifier(crypto, getPubKeyRSA(), signatureAlgorithm);
+
+        case SignatureAlgorithm.rsa_pss_pss_sha256:
+        case SignatureAlgorithm.rsa_pss_pss_sha384:
+        case SignatureAlgorithm.rsa_pss_pss_sha512:
+            validateRSA_PSS_PSS(signatureAlgorithm);
+            return new JcaTlsRSAPSSVerifier(crypto, getPubKeyRSA(), signatureAlgorithm);
 
         default:
             throw new TlsFatalAlert(AlertDescription.certificate_unknown);
         }
     }
 
-    public short getClientCertificateType() throws IOException
-    {
-         PublicKey publicKey = getPublicKey();
-
-         try
-         {
-             /*
-              * TODO RFC 5246 7.4.6. The certificates MUST be signed using an acceptable hash/
-              * signature algorithm pair, as described in Section 7.4.4. Note that this relaxes the
-              * constraints on certificate-signing algorithms found in prior versions of TLS.
-              */
-
-             /*
-              * RFC 5246 7.4.6. Client Certificate
-              */
-
-             /*
-              * RSA public key; the certificate MUST allow the key to be used for signing with the
-              * signature scheme and hash algorithm that will be employed in the certificate verify
-              * message.
-              */
-             if (publicKey instanceof RSAPublicKey)
-             {
-                 validateKeyUsage(KeyUsage.digitalSignature);
-                 return ClientCertificateType.rsa_sign;
-             }
-
-             /*
-              * DSA public key; the certificate MUST allow the key to be used for signing with the
-              * hash algorithm that will be employed in the certificate verify message.
-              */
-             if (publicKey instanceof DSAPublicKey)
-             {
-                 validateKeyUsage(KeyUsage.digitalSignature);
-                 return ClientCertificateType.dss_sign;
-             }
-
-             /*
-              * ECDSA-capable public key; the certificate MUST allow the key to be used for signing
-              * with the hash algorithm that will be employed in the certificate verify message; the
-              * public key MUST use a curve and point format supported by the server.
-              */
-             if (publicKey instanceof ECPublicKey)
-             {
-                 validateKeyUsage(KeyUsage.digitalSignature);
-                 // TODO Check the curve and point format
-                 return ClientCertificateType.ecdsa_sign;
-             }
-
-             // TODO Add support for ClientCertificateType.*_fixed_*
-         }
-         catch (IOException e)
-         {
-             throw e;
-         }
-         catch (Exception e)
-         {
-             throw new TlsFatalAlert(AlertDescription.unsupported_certificate, e);
-         }
-
-         throw new TlsFatalAlert(AlertDescription.unsupported_certificate);
-    }
-
     public byte[] getEncoded() throws IOException
     {
         try
@@ -209,62 +167,169 @@ public class JcaTlsCertificate
 
     DHPublicKey getPubKeyDH() throws IOException
     {
-        DHPublicKey pubKeyDH;
         try
         {
-            pubKeyDH = (DHPublicKey)getPublicKey();
+            return (DHPublicKey)getPublicKey();
         }
         catch (ClassCastException e)
         {
             throw new TlsFatalAlert(AlertDescription.certificate_unknown, e);
         }
-
-        return validatePubKeyDH(pubKeyDH);
     }
 
     DSAPublicKey getPubKeyDSS() throws IOException
     {
-        DSAPublicKey pubKeyDSS;
         try
         {
-            pubKeyDSS = (DSAPublicKey)getPublicKey();
+            return (DSAPublicKey)getPublicKey();
         }
         catch (ClassCastException e)
         {
             throw new TlsFatalAlert(AlertDescription.certificate_unknown, e);
         }
-
-        return validatePubKeyDSS(pubKeyDSS);
     }
 
     ECPublicKey getPubKeyEC() throws IOException
     {
-        ECPublicKey pubKeyEC;
         try
         {
-            pubKeyEC = (ECPublicKey)getPublicKey();
+            return (ECPublicKey)getPublicKey();
         }
         catch (ClassCastException e)
         {
             throw new TlsFatalAlert(AlertDescription.certificate_unknown, e);
         }
+    }
 
-        return validatePubKeyEC(pubKeyEC);
+    PublicKey getPubKeyEd25519() throws IOException
+    {
+        PublicKey publicKey = getPublicKey();
+        if (!"Ed25519".equals(publicKey.getAlgorithm()))
+        {
+            throw new TlsFatalAlert(AlertDescription.certificate_unknown);
+        }
+        return publicKey;
+    }
+
+    PublicKey getPubKeyEd448() throws IOException
+    {
+        PublicKey publicKey = getPublicKey();
+        if (!"Ed448".equals(publicKey.getAlgorithm()))
+        {
+            throw new TlsFatalAlert(AlertDescription.certificate_unknown);
+        }
+        return publicKey;
     }
 
     RSAPublicKey getPubKeyRSA() throws IOException
     {
-        RSAPublicKey pubKeyRSA;
         try
         {
-            pubKeyRSA = (RSAPublicKey)getPublicKey();
+            return (RSAPublicKey)getPublicKey();
         }
         catch (ClassCastException e)
         {
             throw new TlsFatalAlert(AlertDescription.certificate_unknown, e);
         }
+    }
 
-        return validatePubKeyRSA(pubKeyRSA);
+    public short getLegacySignatureAlgorithm() throws IOException
+    {
+         PublicKey publicKey = getPublicKey();
+
+         try
+         {
+             validateKeyUsage(KeyUsage.digitalSignature);
+         }
+         catch (IOException e)
+         {
+             throw e;
+         }
+         catch (Exception e)
+         {
+             throw new TlsFatalAlert(AlertDescription.unsupported_certificate, e);
+         }
+
+         /*
+          * RFC 5246 7.4.6. Client Certificate
+          */
+
+         /*
+          * RSA public key; the certificate MUST allow the key to be used for signing with the
+          * signature scheme and hash algorithm that will be employed in the certificate verify
+          * message.
+          */
+         if (publicKey instanceof RSAPublicKey)
+         {
+             return SignatureAlgorithm.rsa;
+         }
+
+         /*
+          * DSA public key; the certificate MUST allow the key to be used for signing with the
+          * hash algorithm that will be employed in the certificate verify message.
+          */
+         if (publicKey instanceof DSAPublicKey)
+         {
+             return SignatureAlgorithm.dsa;
+         }
+
+         /*
+          * ECDSA-capable public key; the certificate MUST allow the key to be used for signing
+          * with the hash algorithm that will be employed in the certificate verify message; the
+          * public key MUST use a curve and point format supported by the server.
+          */
+         if (publicKey instanceof ECPublicKey)
+         {
+             // TODO Check the curve and point format
+             return SignatureAlgorithm.ecdsa;
+         }
+
+         throw new TlsFatalAlert(AlertDescription.unsupported_certificate);
+    }
+
+    public boolean supportsSignatureAlgorithm(short signatureAlgorithm)
+        throws IOException
+    {
+        if (!supportsKeyUsage(KeyUsage.digitalSignature))
+        {
+            return false;
+        }
+
+        PublicKey publicKey = getPublicKey();
+
+        switch (signatureAlgorithm)
+        {
+        case SignatureAlgorithm.rsa:
+            return supportsRSA_PKCS1()
+                && publicKey instanceof RSAPublicKey;
+
+        case SignatureAlgorithm.dsa:
+            return publicKey instanceof DSAPublicKey;
+
+        case SignatureAlgorithm.ecdsa:
+            return publicKey instanceof ECPublicKey;
+
+        case SignatureAlgorithm.ed25519:
+            return "Ed25519".equals(publicKey.getAlgorithm());
+
+        case SignatureAlgorithm.ed448:
+            return "Ed448".equals(publicKey.getAlgorithm());
+
+        case SignatureAlgorithm.rsa_pss_rsae_sha256:
+        case SignatureAlgorithm.rsa_pss_rsae_sha384:
+        case SignatureAlgorithm.rsa_pss_rsae_sha512:
+            return supportsRSA_PSS_RSAE()
+                && publicKey instanceof RSAPublicKey;
+
+        case SignatureAlgorithm.rsa_pss_pss_sha256:
+        case SignatureAlgorithm.rsa_pss_pss_sha384:
+        case SignatureAlgorithm.rsa_pss_pss_sha512:
+            return supportsRSA_PSS_PSS(signatureAlgorithm)
+                && publicKey instanceof RSAPublicKey;
+
+        default:
+            return false;
+        }
     }
 
     public TlsCertificate useInRole(int connectionEnd, int keyExchangeAlgorithm) throws IOException
@@ -317,13 +382,17 @@ public class JcaTlsCertificate
         }
     }
 
+    protected SubjectPublicKeyInfo getSubjectPublicKeyInfo() throws IOException
+    {
+        return SubjectPublicKeyInfo.getInstance(getPublicKey().getEncoded());
+    }
+
     public X509Certificate getX509Certificate()
     {
         return certificate;
     }
 
-    protected void validateKeyUsage(int keyUsageBits)
-        throws IOException
+    protected boolean supportsKeyUsage(int keyUsageBits)
     {
         Extensions exts;
         try
@@ -332,7 +401,7 @@ public class JcaTlsCertificate
         }
         catch (CertificateEncodingException e)
         {
-            throw new TlsCryptoException("unable to parse certificate extensions: " + e.getMessage(), e);
+            return false;
         }
 
         if (exts != null)
@@ -343,32 +412,68 @@ public class JcaTlsCertificate
                 int bits = ku.getBytes()[0] & 0xff;
                 if ((bits & keyUsageBits) != keyUsageBits)
                 {
-                    throw new TlsFatalAlert(AlertDescription.certificate_unknown);
+                    return false;
                 }
             }
         }
+
+        return true;
     }
 
-    protected DHPublicKey validatePubKeyDH(DHPublicKey pubKeyDH) throws IOException
+    protected boolean supportsRSA_PKCS1()
+        throws IOException
     {
-        return pubKeyDH; // TODO: TlsDHUtils.validateDHPublicKey(pubKeyDH);
+        AlgorithmIdentifier pubKeyAlgID = getSubjectPublicKeyInfo().getAlgorithm();
+        return RSAUtil.supportsPKCS1(pubKeyAlgID);
     }
 
-    protected DSAPublicKey validatePubKeyDSS(DSAPublicKey pubKeyDSS) throws IOException
+    protected boolean supportsRSA_PSS_PSS(short signatureAlgorithm)
+        throws IOException
     {
-        // TODO[tls-ops]
-        return pubKeyDSS;
+        AlgorithmIdentifier pubKeyAlgID = getSubjectPublicKeyInfo().getAlgorithm();
+        return RSAUtil.supportsPSS_PSS(signatureAlgorithm, pubKeyAlgID);
     }
 
-    protected ECPublicKey validatePubKeyEC(ECPublicKey pubKeyEC) throws IOException
+    protected boolean supportsRSA_PSS_RSAE()
+        throws IOException
     {
-        // TODO[tls-ops]
-        return pubKeyEC;
+        AlgorithmIdentifier pubKeyAlgID = getSubjectPublicKeyInfo().getAlgorithm();
+        return RSAUtil.supportsPSS_RSAE(pubKeyAlgID);
     }
 
-    protected RSAPublicKey validatePubKeyRSA(RSAPublicKey pubKeyRSA) throws IOException
+    protected void validateKeyUsage(int keyUsageBits)
+        throws IOException
     {
-        // TODO[tls-ops]
-        return pubKeyRSA;
+        if (!supportsKeyUsage(keyUsageBits))
+        {
+            throw new TlsFatalAlert(AlertDescription.certificate_unknown);
+        }
+    }
+
+    protected void validateRSA_PKCS1()
+        throws IOException
+    {
+        if (!supportsRSA_PKCS1())
+        {
+            throw new TlsFatalAlert(AlertDescription.certificate_unknown);
+        }
+    }
+
+    protected void validateRSA_PSS_PSS(short signatureAlgorithm)
+        throws IOException
+    {
+        if (!supportsRSA_PSS_PSS(signatureAlgorithm))
+        {
+            throw new TlsFatalAlert(AlertDescription.certificate_unknown);
+        }
+    }
+
+    protected void validateRSA_PSS_RSAE()
+        throws IOException
+    {
+        if (!supportsRSA_PSS_RSAE())
+        {
+            throw new TlsFatalAlert(AlertDescription.certificate_unknown);
+        }
     }
 }
