@@ -2,19 +2,16 @@ package org.bouncycastle.tls.crypto.impl.jcajce;
 
 import java.io.IOException;
 import java.math.BigInteger;
-import java.security.AlgorithmParameters;
 import java.security.GeneralSecurityException;
+import java.security.NoSuchAlgorithmException;
 import java.security.PrivateKey;
 import java.security.PublicKey;
 import java.security.SecureRandom;
 import java.security.interfaces.RSAPublicKey;
-import java.security.spec.ECGenParameterSpec;
-import java.security.spec.ECParameterSpec;
 import java.util.Hashtable;
 
 import javax.crypto.Cipher;
 import javax.crypto.KeyAgreement;
-import javax.crypto.SecretKey;
 import javax.crypto.spec.SecretKeySpec;
 
 import org.bouncycastle.asn1.x509.KeyUsage;
@@ -25,6 +22,7 @@ import org.bouncycastle.tls.HashAlgorithm;
 import org.bouncycastle.tls.MACAlgorithm;
 import org.bouncycastle.tls.NamedGroup;
 import org.bouncycastle.tls.ProtocolVersion;
+import org.bouncycastle.tls.SignatureAlgorithm;
 import org.bouncycastle.tls.SignatureAndHashAlgorithm;
 import org.bouncycastle.tls.TlsFatalAlert;
 import org.bouncycastle.tls.TlsUtils;
@@ -116,13 +114,26 @@ public class JcaTlsCrypto
         return entropySource;
     }
 
-    public SecretKey calculateKeyAgreement(String agreementAlgorithm, PrivateKey privateKey, PublicKey publicKey, String secretAlgorithm)
+    public byte[] calculateKeyAgreement(String agreementAlgorithm, PrivateKey privateKey, PublicKey publicKey, String secretAlgorithm)
         throws GeneralSecurityException
     {
         KeyAgreement agreement = helper.createKeyAgreement(agreementAlgorithm);
         agreement.init(privateKey);
         agreement.doPhase(publicKey, true);
-        return agreement.generateSecret(secretAlgorithm);
+
+        try
+        {
+            return agreement.generateSecret(secretAlgorithm).getEncoded();
+        }
+        catch (NoSuchAlgorithmException e)
+        {
+            // Oracle provider currently does not support generateSecret(algorithmName) for these.
+            if ("X25519".equals(agreementAlgorithm) || "X448".equals(agreementAlgorithm))
+            {
+                return agreement.generateSecret();
+            }
+            throw e;
+        }
     }
 
     public TlsCertificate createCertificate(byte[] encoding)
@@ -302,7 +313,10 @@ public class JcaTlsCrypto
 
     public boolean hasAllRawSignatureAlgorithms()
     {
-        return !JcaUtils.isSunMSCAPIProviderActive();
+        // TODO[RFC 8422] Revisit the need to buffer the handshake for "Intrinsic" hash signatures
+        return !JcaUtils.isSunMSCAPIProviderActive()
+            && !hasSignatureAlgorithm(SignatureAlgorithm.ed25519)
+            && !hasSignatureAlgorithm(SignatureAlgorithm.ed448);
     }
 
     public boolean hasDHAgreement()
@@ -395,13 +409,13 @@ public class JcaTlsCrypto
             return false;
         }
 
-        String curveName = NamedGroup.getName(namedGroup);
-        if (curveName == null)
+        String groupName = NamedGroup.getName(namedGroup);
+        if (groupName == null)
         {
             return false;
         }
 
-        int key = Integers.valueOf(namedGroup);
+        Integer key = Integers.valueOf(namedGroup);
 
         synchronized (supportedGroups)
         {
@@ -412,7 +426,36 @@ public class JcaTlsCrypto
             }
         }
 
-        boolean result = isCurveSupported(curveName);
+        boolean result = true;
+        try
+        {
+            switch (namedGroup)
+            {
+            case NamedGroup.x25519:
+            {
+                helper.createKeyFactory("X25519");
+                helper.createKeyPairGenerator("X25519");
+                helper.createKeyAgreement("X25519");
+                break;
+            }
+            case NamedGroup.x448:
+            {
+                helper.createKeyFactory("X448");
+                helper.createKeyPairGenerator("X448");
+                helper.createKeyAgreement("X448");
+                break;
+            }
+            default:
+            {
+                result &= isCurveSupported(groupName);
+                break;
+            }
+            }
+        }
+        catch (GeneralSecurityException e)
+        {
+            result = false;
+        }
 
         synchronized (supportedGroups)
         {
@@ -435,10 +478,25 @@ public class JcaTlsCrypto
         }
     }
 
-    public boolean hasSignatureAlgorithm(int signatureAlgorithm)
+    public boolean hasSignatureAlgorithm(short signatureAlgorithm)
     {
-        // TODO: expand
-        return true;
+        switch (signatureAlgorithm)
+        {
+        case SignatureAlgorithm.rsa:
+        case SignatureAlgorithm.dsa:
+        case SignatureAlgorithm.ecdsa:
+        case SignatureAlgorithm.ed25519:
+        case SignatureAlgorithm.ed448:
+        case SignatureAlgorithm.rsa_pss_rsae_sha256:
+        case SignatureAlgorithm.rsa_pss_rsae_sha384:
+        case SignatureAlgorithm.rsa_pss_rsae_sha512:
+        case SignatureAlgorithm.rsa_pss_pss_sha256:
+        case SignatureAlgorithm.rsa_pss_pss_sha384:
+        case SignatureAlgorithm.rsa_pss_pss_sha512:
+            return true;
+        default:
+            return false;
+        }
     }
 
     public boolean hasSignatureAndHashAlgorithm(SignatureAndHashAlgorithm sigAndHashAlgorithm)
@@ -451,8 +509,7 @@ public class JcaTlsCrypto
             return false;
         }
 
-        // TODO: expand
-        return true;
+        return hasSignatureAlgorithm(sigAndHashAlgorithm.getSignature());
     }
 
     public boolean hasSRPAuthentication()
@@ -492,7 +549,7 @@ public class JcaTlsCrypto
         }
         catch (GeneralSecurityException e)
         {
-            throw new IllegalArgumentException("unable to create message digest:" + e.getMessage(), e);
+            throw Exceptions.illegalArgumentException("unable to create message digest:" + e.getMessage(), e);
         }
     }
 
@@ -503,7 +560,15 @@ public class JcaTlsCrypto
 
     public TlsECDomain createECDomain(TlsECConfig ecConfig)
     {
-        return new JceTlsECDomain(this, ecConfig);
+        switch (ecConfig.getNamedGroup())
+        {
+        case NamedGroup.x25519:
+            return new JceX25519Domain(this);
+        case NamedGroup.x448:
+            return new JceX448Domain(this);
+        default:
+            return new JceTlsECDomain(this, ecConfig);
+        }
     }
 
     public TlsEncryptor createEncryptor(TlsCertificate certificate)
@@ -530,9 +595,17 @@ public class JcaTlsCrypto
                     }
                     catch (Exception e)
                     {
-                        // okay, maybe the provider does not support wrap mode.
-                        c.init(Cipher.ENCRYPT_MODE, pubKeyRSA, getSecureRandom());
-                        return c.doFinal(input, inOff, length);
+                        try
+                        {
+                            // okay, maybe the provider does not support wrap mode.
+                            c.init(Cipher.ENCRYPT_MODE, pubKeyRSA, getSecureRandom());
+                            return c.doFinal(input, inOff, length);
+                        }
+                        catch (Exception ex)
+                        {
+                            // okay, if we get here let's rethrow the original one.
+                            throw new TlsFatalAlert(AlertDescription.internal_error, e);
+                        }
                     }
                 }
                 catch (GeneralSecurityException e)
@@ -559,7 +632,7 @@ public class JcaTlsCrypto
     protected TlsAEADCipherImpl createAEADCipher(String cipherName, String algorithm, int keySize, boolean isEncrypting)
         throws GeneralSecurityException
     {
-        return new JceAEADCipherImpl(helper.createCipher(cipherName), algorithm, isEncrypting);
+        return new JceAEADCipherImpl(helper, cipherName, algorithm, keySize, isEncrypting);
     }
 
     /**
@@ -575,7 +648,7 @@ public class JcaTlsCrypto
     protected TlsBlockCipherImpl createBlockCipher(String cipherName, String algorithm, int keySize, boolean isEncrypting)
         throws GeneralSecurityException
     {
-        return new JceBlockCipherImpl(helper.createCipher(cipherName), algorithm, isEncrypting);
+        return new JceBlockCipherImpl(helper.createCipher(cipherName), algorithm, keySize, isEncrypting);
     }
 
     /**
@@ -641,23 +714,10 @@ public class JcaTlsCrypto
 
     protected boolean isCurveSupported(String curveName)
     {
-        try
-        {
-            AlgorithmParameters params = getHelper().createAlgorithmParameters("EC");
-            params.init(new ECGenParameterSpec(curveName));
-            if (params.getParameterSpec(ECParameterSpec.class) != null)
-            {
-                return true;
-            }
-        }
-        catch (Exception e)
-        {
-        }
-
-        return false;
+        return ECUtil.isCurveSupported(curveName, this.getHelper());
     }
 
-    JcaJceHelper getHelper()
+    public JcaJceHelper getHelper()
     {
         return helper;
     }
@@ -783,7 +843,7 @@ public class JcaTlsCrypto
             digestName = "SHA-512";
             break;
         default:
-            throw new IllegalArgumentException("unknown HashAlgorithm: " + HashAlgorithm.getText(hashAlgorithm));
+            throw new IllegalArgumentException("invalid HashAlgorithm: " + HashAlgorithm.getText(hashAlgorithm));
         }
         return digestName;
     }
