@@ -3,6 +3,7 @@ package org.bouncycastle.tls.test;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
+import java.io.PipedInputStream;
 import java.security.GeneralSecurityException;
 import java.security.KeyFactory;
 import java.security.NoSuchAlgorithmException;
@@ -16,6 +17,7 @@ import javax.net.ssl.KeyManagerFactory;
 import javax.net.ssl.TrustManagerFactory;
 
 import org.bouncycastle.asn1.ASN1ObjectIdentifier;
+import org.bouncycastle.asn1.edec.EdECObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.PrivateKeyInfo;
 import org.bouncycastle.asn1.pkcs.RSAPrivateKey;
@@ -26,7 +28,6 @@ import org.bouncycastle.crypto.digests.SHA256Digest;
 import org.bouncycastle.crypto.params.AsymmetricKeyParameter;
 import org.bouncycastle.crypto.params.RSAPrivateCrtKeyParameters;
 import org.bouncycastle.crypto.util.PrivateKeyFactory;
-import org.bouncycastle.jce.provider.BouncyCastleProvider;
 import org.bouncycastle.tls.AlertDescription;
 import org.bouncycastle.tls.Certificate;
 import org.bouncycastle.tls.SignatureAlgorithm;
@@ -107,15 +108,33 @@ public class TlsTestUtils
     }
 
     static String getCACertResource(short signatureAlgorithm) throws IOException
+    {
+        return "x509-ca-" + getResourceName(signatureAlgorithm) + ".pem";
+    }
+
+    static String getResourceName(short signatureAlgorithm) throws IOException
     {
         switch (signatureAlgorithm)
         {
-        case SignatureAlgorithm.dsa:
-            return "x509-ca-dsa.pem";
-        case SignatureAlgorithm.ecdsa:
-            return "x509-ca-ecdsa.pem";
         case SignatureAlgorithm.rsa:
-            return "x509-ca-rsa.pem";
+        case SignatureAlgorithm.rsa_pss_rsae_sha256:
+        case SignatureAlgorithm.rsa_pss_rsae_sha384:
+        case SignatureAlgorithm.rsa_pss_rsae_sha512:
+            return "rsa";
+        case SignatureAlgorithm.dsa:
+            return "dsa";
+        case SignatureAlgorithm.ecdsa:
+            return "ecdsa";
+        case SignatureAlgorithm.ed25519:
+            return "ed25519";
+        case SignatureAlgorithm.ed448:
+            return "ed448";
+        case SignatureAlgorithm.rsa_pss_pss_sha256:
+            return "rsa_pss_256";
+        case SignatureAlgorithm.rsa_pss_pss_sha384:
+            return "rsa_pss_384";
+        case SignatureAlgorithm.rsa_pss_pss_sha512:
+            return "rsa_pss_512";
         default:
             throw new TlsFatalAlert(AlertDescription.internal_error);
         }
@@ -132,13 +151,14 @@ public class TlsTestUtils
         {
             AsymmetricKeyParameter privateKey = loadBcPrivateKeyResource(keyResource);
 
-            return new BcDefaultTlsCredentialedAgreement((BcTlsCrypto)context.getCrypto(), certificate, privateKey);
+            return new BcDefaultTlsCredentialedAgreement((BcTlsCrypto)crypto, certificate, privateKey);
         }
         else
         {
-            PrivateKey privateKey = loadJcaPrivateKeyResource(keyResource);
+            JcaTlsCrypto jcaCrypto = (JcaTlsCrypto)crypto;
+            PrivateKey privateKey = loadJcaPrivateKeyResource(jcaCrypto, keyResource);
 
-            return new JceDefaultTlsCredentialedAgreement((JcaTlsCrypto)context.getCrypto(), certificate, privateKey);
+            return new JceDefaultTlsCredentialedAgreement(jcaCrypto, certificate, privateKey);
         }
     }
 
@@ -157,9 +177,10 @@ public class TlsTestUtils
         }
         else
         {
-            PrivateKey privateKey = loadJcaPrivateKeyResource(keyResource);
+            JcaTlsCrypto jcaCrypto = (JcaTlsCrypto)crypto;
+            PrivateKey privateKey = loadJcaPrivateKeyResource(jcaCrypto, keyResource);
 
-            return new JceDefaultTlsCredentialedDecryptor((JcaTlsCrypto)crypto, certificate, privateKey);
+            return new JceDefaultTlsCredentialedDecryptor(jcaCrypto, certificate, privateKey);
         }
     }
 
@@ -168,19 +189,21 @@ public class TlsTestUtils
     {
         TlsCrypto crypto = context.getCrypto();
         Certificate certificate = loadCertificateChain(crypto, certResources);
+        TlsCryptoParameters cryptoParams = new TlsCryptoParameters(context);
 
         // TODO[tls-ops] Need to have TlsCrypto construct the credentials from the certs/key (as raw data)
         if (crypto instanceof BcTlsCrypto)
         {
             AsymmetricKeyParameter privateKey = loadBcPrivateKeyResource(keyResource);
 
-            return new BcDefaultTlsCredentialedSigner(new TlsCryptoParameters(context), (BcTlsCrypto)crypto, privateKey, certificate, signatureAndHashAlgorithm);
+            return new BcDefaultTlsCredentialedSigner(cryptoParams, (BcTlsCrypto)crypto, privateKey, certificate, signatureAndHashAlgorithm);
         }
         else
         {
-            PrivateKey privateKey = loadJcaPrivateKeyResource(keyResource);
+            JcaTlsCrypto jcaCrypto = (JcaTlsCrypto)crypto;
+            PrivateKey privateKey = loadJcaPrivateKeyResource(jcaCrypto, keyResource);
 
-            return new JcaDefaultTlsCredentialedSigner(new TlsCryptoParameters(context), (JcaTlsCrypto)crypto, privateKey, certificate, signatureAndHashAlgorithm);
+            return new JcaDefaultTlsCredentialedSigner(cryptoParams, jcaCrypto, privateKey, certificate, signatureAndHashAlgorithm);
         }
     }
 
@@ -214,6 +237,27 @@ public class TlsTestUtils
             keyResource, signatureAndHashAlgorithm);
     }
 
+    static TlsCredentialedSigner loadSignerCredentialsServer(TlsContext context, Vector supportedSignatureAlgorithms,
+        short signatureAlgorithm) throws IOException
+    {
+        String sigName = getResourceName(signatureAlgorithm);
+
+        switch (signatureAlgorithm)
+        {
+        case SignatureAlgorithm.rsa:
+        case SignatureAlgorithm.rsa_pss_rsae_sha256:
+        case SignatureAlgorithm.rsa_pss_rsae_sha384:
+        case SignatureAlgorithm.rsa_pss_rsae_sha512:
+            sigName += "-sign";
+            break;
+        }
+
+        String certResource = "x509-server-" + sigName + ".pem";
+        String keyResource = "x509-server-key-" + sigName + ".pem";
+
+        return loadSignerCredentials(context, supportedSignatureAlgorithms, signatureAlgorithm, certResource, keyResource);
+    }
+
     static Certificate loadCertificateChain(TlsCrypto crypto, String[] resources)
         throws IOException
     {
@@ -276,7 +320,7 @@ public class TlsTestUtils
         throw new IllegalArgumentException("'resource' doesn't specify a valid private key");
     }
 
-    static PrivateKey loadJcaPrivateKeyResource(String resource)
+    static PrivateKey loadJcaPrivateKeyResource(JcaTlsCrypto crypto, String resource)
         throws IOException
     {
         Throwable cause = null;
@@ -285,7 +329,7 @@ public class TlsTestUtils
             PemObject pem = loadPemResource(resource);
             if (pem.getType().equals("PRIVATE KEY"))
             {
-                return loadJcaPkcs8PrivateKey(pem.getContent());
+                return loadJcaPkcs8PrivateKey(crypto, pem.getContent());
             }
             if (pem.getType().equals("ENCRYPTED PRIVATE KEY"))
             {
@@ -294,10 +338,10 @@ public class TlsTestUtils
             if (pem.getType().equals("RSA PRIVATE KEY"))
             {
                 RSAPrivateKey rsa = RSAPrivateKey.getInstance(pem.getContent());
-                    KeyFactory keyFact = KeyFactory.getInstance("RSA", BouncyCastleProvider.PROVIDER_NAME);
-                    return keyFact.generatePrivate(new RSAPrivateCrtKeySpec(rsa.getModulus(), rsa.getPublicExponent(),
-                        rsa.getPrivateExponent(), rsa.getPrime1(), rsa.getPrime2(), rsa.getExponent1(),
-                        rsa.getExponent2(), rsa.getCoefficient()));
+                KeyFactory keyFact = crypto.getHelper().createKeyFactory("RSA");
+                return keyFact.generatePrivate(new RSAPrivateCrtKeySpec(rsa.getModulus(), rsa.getPublicExponent(),
+                    rsa.getPrivateExponent(), rsa.getPrime1(), rsa.getPrime2(), rsa.getExponent1(), rsa.getExponent2(),
+                    rsa.getCoefficient()));
             }
         }
         catch (GeneralSecurityException e)
@@ -307,7 +351,7 @@ public class TlsTestUtils
         throw new IllegalArgumentException("'resource' doesn't specify a valid private key", cause);
     }
 
-    static PrivateKey loadJcaPkcs8PrivateKey(byte[] encoded) throws GeneralSecurityException
+    static PrivateKey loadJcaPkcs8PrivateKey(JcaTlsCrypto crypto, byte[] encoded) throws GeneralSecurityException
     {
         PrivateKeyInfo pki = PrivateKeyInfo.getInstance(encoded);
         AlgorithmIdentifier algID = pki.getPrivateKeyAlgorithm();
@@ -323,16 +367,25 @@ public class TlsTestUtils
             // TODO Try ECDH/ECDSA according to intended use?
             name = "EC";
         }
-        else if (PKCSObjectIdentifiers.rsaEncryption.equals(oid))
+        else if (PKCSObjectIdentifiers.rsaEncryption.equals(oid)
+            || PKCSObjectIdentifiers.id_RSASSA_PSS.equals(oid))
         {
             name = "RSA";
         }
+        else if (EdECObjectIdentifiers.id_Ed25519.equals(oid))
+        {
+            name = "Ed25519";
+        }
+        else if (EdECObjectIdentifiers.id_Ed448.equals(oid))
+        {
+            name = "Ed448";
+        }
         else
         {
             name = oid.getId();
         }
 
-        KeyFactory kf = KeyFactory.getInstance(name, BouncyCastleProvider.PROVIDER_NAME);
+        KeyFactory kf = crypto.getHelper().createKeyFactory(name);
         return kf.generatePrivate(new PKCS8EncodedKeySpec(encoded));
     }
 
@@ -395,4 +448,18 @@ public class TlsTestUtils
             return KeyManagerFactory.getInstance("SunX509");
         }
     }
+
+    static PipedInputStream createPipedInputStream()
+    {
+        return new BigPipedInputStream(16384);
+    }
+
+    private static class BigPipedInputStream
+        extends PipedInputStream
+    {
+        BigPipedInputStream(int size)
+        {
+            this.buffer = new byte[size];
+        }
+    }
 }
